-- Version: r41
-- Last modified: 2010-05-01T04:45:02Z
-- Last modified by: joselucas

local function print(value)
	if DEFAULT_CHAT_FRAME then
		DEFAULT_CHAT_FRAME:AddMessage(value);
	end

	if SELECTED_CHAT_FRAME and SELECTED_CHAT_FRAME ~= DEFAULT_CHAT_FRAME then
		SELECTED_CHAT_FRAME:AddMessage(value);
	end
end;

local function printf(stringFormat,  ...)
	print(string.format(stringFormat,  ...));
end;

local addon = _G["SimpleDamageLog"];

local me = {
	name = "SimpleDamageLogConfig",
	frameName = "SimpleDamageLogConfigFrame",
	
	checkboxes = {
		"Enable",
		"Lock",
		"OutgoingDamage",
		"OutgoingHeals",
		"IncomingDamage",
		"IncomingHeals",
		"ShowDamageSource",
		"ShowHealSource",
	},
	
	buttons = {
		"OK",
		"Cancel",
		"Apply",
	},
}

function me.OnLoad()
	-- set localized text on all widgets
	local titleText = string.format("%s %s", addon.name, addon.version);
	getglobal(me.frameName .. "TitleText"):SetText(titleText);

	for _, checkbox in ipairs(me.checkboxes) do
		local controlName = me.frameName .. checkbox .. "Text";
		getglobal(controlName):SetText(addon.locale[checkbox]);
	end

	for _, button in ipairs(me.buttons) do
		local controlName = me.frameName .. button;
		getglobal(controlName):SetText(addon.locale[button]);
	end
end

function me.OnShow()
	for _, checkbox in ipairs(me.checkboxes) do
		local controlName = me.frameName .. checkbox;
		getglobal(controlName):SetChecked(addon.config[checkbox]);
	end
end

function me.OnApplySettings()
	for _, checkbox in ipairs(me.checkboxes) do
		local controlName = me.frameName .. checkbox;
		addon.config[checkbox] = getglobal(controlName):IsChecked();
	end
	
	addon.Init();
end

function me.OnEnable(checkbox)
	local enabled = checkbox:IsChecked();
	
	for _, checkbox in ipairs(me.checkboxes) do
		if checkbox ~= "Enable" then
			local controlName = me.frameName .. checkbox;
			
			if enabled then
				getglobal(controlName):Enable();
			else
				getglobal(controlName):Disable();
			end
		end
	end
end

function me.OnIncomingDamage(checkbox)
	if checkbox:IsChecked() then
		getglobal(me.frameName .. "ShowDamageSource"):Enable();
	else
		getglobal(me.frameName .. "ShowDamageSource"):Disable();
	end
end

function me.OnIncomingHeals(checkbox)
	if checkbox:IsChecked() then
		getglobal(me.frameName .. "ShowHealSource"):Enable();
	else
		getglobal(me.frameName .. "ShowHealSource"):Disable();
	end
end

_G[me.name] = me;