
-- Comparison functions used for table sorting

local compareIndexNew = function(a, b)
	return (a.index > b.index)
end
local compareIndexOld = function(a, b)
	return (a.index < b.index)
end

local compareUsVictory = function(a, b)
	local victoryValue = {2, 1, 3}
	local aValue = victoryValue[a.victorIndex + 1]
	local bValue = victoryValue[b.victorIndex + 1]
	return (aValue < bValue)
end
local compareUsDefeat = function(a, b)
	local victoryValue = {2, 1, 3}
	local aValue = victoryValue[a.victorIndex + 1]
	local bValue = victoryValue[b.victorIndex + 1]
	return (aValue > bValue)
end

local compareUsNameLow = function(a, b)
	if (a.teamRecord[1].guildName == b.teamRecord[1].guildName) then
		return (a.teamRecord[1].guildServer < b.teamRecord[1].guildServer)
	end
	return (a.teamRecord[1].guildName < b.teamRecord[1].guildName)
end
local compareUsNameHigh = function(a, b)
	if (a.teamRecord[1].guildName == b.teamRecord[1].guildName) then
		return (a.teamRecord[1].guildServer > b.teamRecord[1].guildServer)
	end
	return (a.teamRecord[1].guildName > b.teamRecord[1].guildName)
end

local compareUsRatingHigh = function(a, b)
	local aRating = a.teamRecord[1].startRating or -1
	local bRating = b.teamRecord[1].startRating or -1
	return (aRating > bRating)
end
local compareUsRatingLow = function(a, b)
	local aRating = a.teamRecord[1].startRating or -1
	local bRating = b.teamRecord[1].startRating or -1
	return (aRating < bRating)
end

local compareEnemyVictory = function(a, b)
	local victoryValue = {2, 3, 1}
	local aValue = victoryValue[a.victorIndex + 1]
	local bValue = victoryValue[b.victorIndex + 1]
	return (aValue < bValue)
end
local compareEnemyDefeat = function(a, b)
	local victoryValue = {2, 3, 1}
	local aValue = victoryValue[a.victorIndex + 1]
	local bValue = victoryValue[b.victorIndex + 1]
	return (aValue > bValue)
end

local compareEnemyNameLow = function(a, b)
	if (a.teamRecord[2].guildName == b.teamRecord[2].guildName) then
		return (a.teamRecord[2].guildServer < b.teamRecord[2].guildServer)
	end
	return (a.teamRecord[2].guildName < b.teamRecord[2].guildName)
end
local compareEnemyNameHigh = function(a, b)
	if (a.teamRecord[2].guildName == b.teamRecord[2].guildName) then
		return (a.teamRecord[2].guildServer > b.teamRecord[2].guildServer)
	end
	return (a.teamRecord[2].guildName > b.teamRecord[2].guildName)
end

local compareEnemyRatingHigh = function(a, b)
	local aRating = a.teamRecord[2].startRating or -1
	local bRating = b.teamRecord[2].startRating or -1
	return (aRating > bRating)
end
local compareEnemyRatingLow = function(a, b)
	local aRating = a.teamRecord[2].startRating or -1
	local bRating = b.teamRecord[2].startRating or -1
	return (aRating < bRating)
end

local sortBattleList = {
	{id = 1, compFunc1 = compareIndexNew, compFunc2 = compareIndexOld},
	{id = 2, compFunc1 = compareUsVictory, compFunc2 = compareUsDefeat},
	{id = 3, compFunc1 = compareUsNameLow, compFunc2 = compareUsNameHigh},
	{id = 4, compFunc1 = compareUsRatingHigh, compFunc2 = compareUsRatingLow},
	{id = 5, compFunc1 = compareEnemyVictory, compFunc2 = compareEnemyDefeat},
	{id = 6, compFunc1 = compareEnemyNameLow, compFunc2 = compareEnemyNameHigh},
	{id = 7, compFunc1 = compareEnemyRatingHigh, compFunc2 = compareEnemyRatingLow},
}

local currentBattleListSortFunction = compareIndexNew

--------------------------------------------------------------------------------

-- height ~24
local MAX_BATTLELIST_LIST_ITEMS = 11
local sortableBattleHistory = {}

WarRecord.Locale.BATTLE_HISTORY_TITLE = "War Record battle history"
WarRecord.Locale.SETTINGS = "Settings"
WarRecord.Locale.CONTEXT_DELETE_ENTRY = "Delete entry"
WarRecord.Locale.COLUMN_BATTLE_ID = "Index"
WarRecord.Locale.COLUMN_BATTLE_GUILD = "Guild"
WarRecord.Locale.COLUMN_BATTLE_SERVER = "(Server)"
WarRecord.Locale.COLUMN_BATTLE_RATING = "Rating"

--------------------------------------------------------------------------------

function WarRecord.BattleListFrame_OnLoad(frame)
	UIPanelBackdropFrame_SetTexture(frame, "Interface/Common/PanelCommonFrame", 256, 256)
	getglobal(frame:GetName() .. "Title"):SetText(WarRecord.Locale.BATTLE_HISTORY_TITLE)
	BattleListFrame_ConfigButton:SetText(WarRecord.Locale.SETTINGS)

	local info = {}
	info.columnTemplate = "BattleList_ColumnTemplate"
	info.itemTemplate = "BattleList_ItemTemplate"
	info.itemCount = MAX_BATTLELIST_LIST_ITEMS
	info.itemHorizontalLine = 1
	info.callbackFunc_GetCount = WarRecord.BattleList_ListFrame_CallbackFunc_GetCount
	info.callbackFunc_UpdateItme = WarRecord.BattleList_ListFrame_CallbackFunc_UpdateItem
	WarRecord.ListScrollInit(BattleList_ListFrame, info)
	UIPanelCommonListFrame_Init(BattleList_ListFrame, info)

	-- column headers
	getglobal("BattleList_ListFrame_Common_Index_Overlay_Text"):SetText(WarRecord.Locale.COLUMN_BATTLE_ID)
	getglobal("BattleList_ListFrame_Common_UsGuild_Overlay_Name"):SetText(WarRecord.Locale.COLUMN_BATTLE_GUILD)
	getglobal("BattleList_ListFrame_Common_UsGuild_Overlay_Server"):SetText(WarRecord.Locale.COLUMN_BATTLE_SERVER)
	getglobal("BattleList_ListFrame_Common_UsRating_Overlay_Text"):SetText(WarRecord.Locale.COLUMN_BATTLE_RATING)
	getglobal("BattleList_ListFrame_Common_EnemyGuild_Overlay_Name"):SetText(WarRecord.Locale.COLUMN_BATTLE_GUILD)
	getglobal("BattleList_ListFrame_Common_EnemyGuild_Overlay_Server"):SetText(WarRecord.Locale.COLUMN_BATTLE_SERVER)
	getglobal("BattleList_ListFrame_Common_EnemyRating_Overlay_Text"):SetText(WarRecord.Locale.COLUMN_BATTLE_RATING)

	local i, v
	for i, v in ipairs(BattleList_ListFrame.items) do
		UIDropDownMenu_Initialize(v, WarRecord.BattleListContextMenu_CallbackShow)
	end
end


function WarRecord.BattleListItem_OnLoad(frame)
	frame:RegisterForClicks("LeftButton", "RightButton")
end

--------------------------------------------------------------------------------

-- Create a copy of the battle history table for manipulation.
function WarRecord.BattleList_CreateSortableHistory()
	sortableBattleHistory = {}

	local i, brIndex, sortedIndex
	for i = 1, #(WarRecordProfile.BattleHistory) do
		brIndex = #(WarRecordProfile.BattleHistory) - i + 1
		sortableBattleHistory[i] = {index = brIndex, teamRecord = {}}
		if (WarRecordProfile.BattleHistory[brIndex].selfTeamIndex == 1) then
			sortedIndex = {1, 2}
		else
			sortedIndex = {2, 1}
		end
		sortableBattleHistory[i].teamRecord[1] = WarRecordProfile.BattleHistory[brIndex].teamRecord[sortedIndex[1]]
		sortableBattleHistory[i].teamRecord[2] = WarRecordProfile.BattleHistory[brIndex].teamRecord[sortedIndex[2]]
		if (WarRecordProfile.BattleHistory[brIndex].victorIndex) and (WarRecordProfile.BattleHistory[brIndex].victorIndex > 0) then
			sortableBattleHistory[i].victorIndex = sortedIndex[WarRecordProfile.BattleHistory[brIndex].victorIndex]
		else
			sortableBattleHistory[i].victorIndex = 0
		end
		sortableBattleHistory[i].timestamps = WarRecordProfile.BattleHistory[brIndex].timestamps
	end

	table.sort(sortableBattleHistory, currentBattleListSortFunction)
	BattleList_ListFrame:Update()
end

--------------------------------------------------------------------------------

function WarRecord.BattleList_IndexStr(sBattle)
	local indexStr = nil
	if (sBattle.timestamps) then
		indexStr = WarRecord.FormatTimestamp(WarRecord.AdjustToServerTime(sBattle.timestamps.recorded))
	end
	if (not indexStr) then
		indexStr = tostring(sBattle.index)
	end
	return indexStr
end

--------------------------------------------------------------------------------

function WarRecord.BattleList_ListFrame_CallbackFunc_GetCount(listFrame)
	return #(sortableBattleHistory)
end


function WarRecord.BattleList_ListFrame_CallbackFunc_UpdateItem(listFrame, item, index)
	item:SetID(index)

	local itemName = item:GetName() .. "_Overlay"
	local indexObj = getglobal(itemName .. "_Index")

	local usVictoryIconObj = getglobal(itemName .. "_UsVictoryIcon")
	local usNameObj = getglobal(itemName .. "_UsName")
	local usServerObj = getglobal(itemName .. "_UsServer")
	local usRatingObj = getglobal(itemName .. "_UsRating")

	local enemyVictoryIconObj = getglobal(itemName .. "_EnemyVictoryIcon")
	local enemyNameObj = getglobal(itemName .. "_EnemyName")
	local enemyServerObj = getglobal(itemName .. "_EnemyServer")
	local enemyRatingObj = getglobal(itemName .. "_EnemyRating")

	indexObj:SetText(WarRecord.BattleList_IndexStr(sortableBattleHistory[index]))

	local team
	team = sortableBattleHistory[index].teamRecord[1]
	if not (team) then
		return
	end
	if (team.guildName) and (team.guildServer) then
		usNameObj:SetText(team.guildName)
		usServerObj:SetText(string.format(WarRecord.Locale.FORMAT_SERVER, team.guildServer))
	else
		usNameObj:SetText(WarRecord.Locale.UNKNOWN_STR)
		usServerObj:SetText(string.format(WarRecord.Locale.FORMAT_SERVER, WarRecord.Locale.UNKNOWN_STR))
	end

	if (team.startRating) and (team.startRating ~= -1) then
		usRatingObj:SetText(team.startRating)
	else
		usRatingObj:SetText(WarRecord.Locale.UNKNOWN_STR)
	end

	team = sortableBattleHistory[index].teamRecord[2]
	if not (team) then
		return
	end
	if (team.guildName) and (team.guildServer) then
		enemyNameObj:SetText(team.guildName)
		enemyServerObj:SetText(string.format(WarRecord.Locale.FORMAT_SERVER, team.guildServer))
	else
		enemyNameObj:SetText(WarRecord.Locale.UNKNOWN_STR)
		enemyServerObj:SetText(string.format(WarRecord.Locale.FORMAT_SERVER, WarRecord.Locale.UNKNOWN_STR))
	end

	if (team.startRating) and (team.startRating ~= -1) then
		enemyRatingObj:SetText(team.startRating)
	else
		enemyRatingObj:SetText(WarRecord.Locale.UNKNOWN_STR)
	end

	if (sortableBattleHistory[index].victorIndex == 1) then
		usVictoryIconObj:SetTexture("Interface\\GuildHouseFrame\\GuildHouseWar_Win")
		enemyVictoryIconObj:SetTexture("Interface\\GuildHouseFrame\\GuildHouseWar_Lost")
	elseif (sortableBattleHistory[index].victorIndex == 2) then
		usVictoryIconObj:SetTexture("Interface\\GuildHouseFrame\\GuildHouseWar_Lost")
		enemyVictoryIconObj:SetTexture("Interface\\GuildHouseFrame\\GuildHouseWar_Win")
	else
		usVictoryIconObj:SetTexture("")
		enemyVictoryIconObj:SetTexture("")
	end
end

--------------------------------------------------------------------------------

function WarRecord.BattleListColumnButton_OnClick(frame)
	local id = frame:GetID()

	-- Find next sort function to use based on column.
	if (currentBattleListSortFunction == sortBattleList[id].compFunc1) then
		currentBattleListSortFunction = sortBattleList[id].compFunc2
	else
		currentBattleListSortFunction = sortBattleList[id].compFunc1
	end
	table.sort(sortableBattleHistory, currentBattleListSortFunction)
	BattleList_ListFrame:Update()
end


function WarRecord.BattleListItem_OnClick(frame, key)
	local id = frame:GetID()

	if (key == "LBUTTON") then
		local brIndex = sortableBattleHistory[id].index

		-- open scoreboard for selected battle
		WarRecord.DisplayCopyCreate(WarRecordProfile.BattleHistory[brIndex])
		WarRecord.ScoreBoardBattleDetailInit(WarRecordProfile.BattleHistory[brIndex])
		getglobal("WarRecordStatusFrameTitle"):SetText(string.format(WarRecord.Locale.FORMAT_SCOREBOARD_TITLE, WarRecord.BattleList_IndexStr(sortableBattleHistory[id])))
		WarRecordStatusFrame:Show()
	elseif (key == "RBUTTON") then
		-- open context menu
		ToggleDropDownMenu(frame, 1, id, "cursor", 0, 0)
	end
end

--------------------------------------------------------------------------------

function WarRecord.BattleListContextMenu_CallbackShow()
	local buttonInfo = {}
	buttonInfo.text = WarRecord.Locale.CONTEXT_DELETE_ENTRY
	buttonInfo.func = WarRecord.BattleListContextMenu_CallbackClickRemove
	buttonInfo.notCheckable = 1
	UIDropDownMenu_AddButton(buttonInfo, 1)
end


function WarRecord.BattleListContextMenu_CallbackClickRemove()
	-- possibly also button:GetParent():GetID()
	local id = UIDROPDOWNMENU_MENU_VALUE
	local brIndex = sortableBattleHistory[id].index

	CloseDropDownMenus()

	table.remove(WarRecordProfile.BattleHistory, brIndex)
	WarRecord.BattleList_CreateSortableHistory()
end

--------------------------------------------------------------------------------

function WarRecord.ConfigButton_OnClick(frame)
	WarRecordConfig:Show()
end

