
-- Comparison functions used for table sorting

local compareNameLess = function(a, b)
	return (a.name < b.name)
end
local compareNameGreater = function(a, b)
	return (a.name > b.name)
end

local compareKillLess = function(a, b)
	return (a.kills < b.kills)
end
local compareKillGreater = function(a, b)
	return (a.kills > b.kills)
end

local compareDamageLess = function(a, b)
	return (a.damage < b.damage)
end
local compareDamageGreater = function(a, b)
	return (a.damage > b.damage)
end

local compareDefendLess = function(a, b)
	return (a.defend < b.defend)
end
local compareDefendGreater = function(a, b)
	return (a.defend > b.defend)
end

local compareHealLess = function(a, b)
	return (a.healing < b.healing)
end
local compareHealGreater = function(a, b)
	return (a.healing > b.healing)
end

local compareMeritLess = function(a, b)
	return (a.merits < b.merits)
end
local compareMeritGreater = function(a, b)
	return (a.merits > b.merits)
end

local compareEnergyLess = function(a, b)
	return (a.energy < b.energy)
end
local compareEnergyGreater = function(a, b)
	return (a.energy > b.energy)
end

local compareOccupyLess = function(a, b)
	return (a.occupations < b.occupations)
end
local compareOccupyGreater = function(a, b)
	return (a.occupations > b.occupations)
end

local compareScoreLess = function(a, b)
	return (a.score < b.score)
end
local compareScoreGreater = function(a, b)
	return (a.score > b.score)
end

local compareTypeLess = function(a, b)
	if (a.unitType == b.unitType) then
		if (a.rank == b.rank) then
			return (a.score > b.score)
		else
			return (a.rank < b.rank)
		end
	else
		return (a.unitType < b.unitType)
	end
end
local compareTypeGreater = function(a, b)
	if (a.unitType == b.unitType) then
		if (a.rank == b.rank) then
			return (a.score < b.score)
		else
			return (a.rank > b.rank)
		end
	else
		return (a.unitType > b.unitType)
	end
end

local compareRankLess = function(a, b)
	if (a.rank == b.rank) then
		if (a.unitType == b.unitType) then
			return (a.score > b.score)
		else
			return (a.unitType < b.unitType)
		end
	else
		return (a.rank < b.rank)
	end
end
local compareRankGreater = function(a, b)
	if (a.rank == b.rank) then
		if (a.unitType == b.unitType) then
			return (a.score < b.score)
		else
			return (a.unitType > b.unitType)
		end
	else
		return (a.rank > b.rank)
	end
end

local compareClassLess = function (a, b)
	if (a.class.primary == b.class.primary) then
		if (a.class.primeLevel == b.class.primeLevel) then
			if (a.class.secondary == b.class.secondary) then
				return (a.class.secondLevel > b.class.secondLevel)
			end
			return (a.class.secondary < b.class.secondary)
		end
		return (a.class.primeLevel > b.class.primeLevel)
	end
	return (a.class.primary < b.class.primary)
end
local compareClassGreater = function (a, b)
	if (a.class.primary == b.class.primary) then
		if (a.class.primeLevel == b.class.primeLevel) then
			if (a.class.secondary == b.class.secondary) then
				return (a.class.secondLevel < b.class.secondLevel)
			end
			return (a.class.secondary > b.class.secondary)
		end
		return (a.class.primeLevel < b.class.primeLevel)
	end
	return (a.class.primary > b.class.primary)
end

-- board page, column button id, compare function, reverse function
local sortScoreBoard = {
	{page = WarRecord.SCORE_PAGE_STATUS, id = 1, compFunc1 = compareNameLess, compFunc2 = compareNameGreater},
	{page = WarRecord.SCORE_PAGE_STATUS, id = 2, compFunc1 = compareKillGreater, compFunc2 = compareKillLess},
	{page = WarRecord.SCORE_PAGE_STATUS, id = 3, compFunc1 = compareDamageGreater, compFunc2 = compareDamageLess},
	{page = WarRecord.SCORE_PAGE_STATUS, id = 4, compFunc1 = compareDefendGreater, compFunc2 = compareDefendLess},
	{page = WarRecord.SCORE_PAGE_STATUS, id = 5, compFunc1 = compareHealGreater, compFunc2 = compareHealLess},
	{page = WarRecord.SCORE_PAGE_STATUS, id = 6, compFunc1 = compareMeritGreater, compFunc2 = compareMeritLess},
	{page = WarRecord.SCORE_PAGE_STATUS, id = 7, compFunc1 = compareEnergyGreater, compFunc2 = compareEnergyLess},
	{page = WarRecord.SCORE_PAGE_STATUS, id = 8, compFunc1 = compareOccupyGreater, compFunc2 = compareOccupyLess},
	{page = WarRecord.SCORE_PAGE_REWARD, id = 1, compFunc1 = compareNameLess, compFunc2 = compareNameGreater},
	{page = WarRecord.SCORE_PAGE_REWARD, id = 2, compFunc1 = compareClassLess, compFunc2 = compareClassGreater},
	{page = WarRecord.SCORE_PAGE_REWARD, id = 3, compFunc1 = compareTypeLess, compFunc2 = compareTypeGreater},
	{page = WarRecord.SCORE_PAGE_REWARD, id = 4, compFunc1 = compareScoreGreater, compFunc2 = compareScoreLess},
	{page = WarRecord.SCORE_PAGE_REWARD, id = 5, compFunc1 = compareRankLess, compFunc2 = compareRankGreater}
}

local currentScoreBoardSortFunction = compareNameLess

--------------------------------------------------------------------------------

-- height of about 26.35 each
local MAX_SCOREBOARD_LIST_ITEMS = 13
local scorePage = WarRecord.SCORE_PAGE_STATUS

WarRecord.Locale.FORMAT_SCOREBOARD_TITLE = "War Record (battle %s)"
WarRecord.Locale.UNKNOWN_STR = "--"
WarRecord.Locale.FORMAT_SERVER = "(%s)"
WarRecord.Locale.FORMAT_PARTICIPANT_SINGLE = "%s participant"
WarRecord.Locale.FORMAT_PARTICIPANTS_PLURAL = "%s participants"
WarRecord.Locale.FORMAT_BATTLE_SCORE = "Score:  %s"
WarRecord.Locale.FORMAT_RATING = "Rating:  %s"
WarRecord.Locale.UNKNOWN_RATING_CHANGE = "(+/-)"
WarRecord.Locale.RATING_NO_CHANGE = "(no change)"
WarRecord.Locale.FORMAT_RATING_CHANGE = "(%+d)"

--------------------------------------------------------------------------------

function WarRecord.ScoreBoardBattleDetailInit(battle)
	WarRecord.ScoreBoardBattleDetailTeamSet(WarRecordStatusFrameBattleDetailsTeamUs, displayData.battle.teamRecord[displayData.usIndex])
	WarRecord.ScoreBoardBattleDetailTeamSet(WarRecordStatusFrameBattleDetailsTeamEnemy, displayData.battle.teamRecord[displayData.enemyIndex])

	local usVictoryIconObj = WarRecordStatusFrameBattleDetailsTeamUsVictoryIcon
	local enemyVictoryIconObj = WarRecordStatusFrameBattleDetailsTeamEnemyVictoryIcon

	if (displayData.battle.victorIndex == displayData.usIndex) then
		usVictoryIconObj:SetTexture("Interface/GuildHouseFrame/GuildHouseWar_Win")
		enemyVictoryIconObj:SetTexture("Interface/GuildHouseFrame/GuildHouseWar_Lost")
	elseif (displayData.battle.victorIndex == displayData.enemyIndex) then
		usVictoryIconObj:SetTexture("Interface/GuildHouseFrame/GuildHouseWar_Lost")
		enemyVictoryIconObj:SetTexture("Interface/GuildHouseFrame/GuildHouseWar_Win")
	else
		usVictoryIconObj:SetTexture("")
		enemyVictoryIconObj:SetTexture("")
	end
end


function WarRecord.ScoreBoardBattleDetailTeamSet(teamFrame, team)
	local teamFrameName = teamFrame:GetName()
	local backgroundBarObj = getglobal(teamFrameName .. "BackgroundBar")
	local nameObj = getglobal(teamFrameName .. "Name")
	local serverObj = getglobal(teamFrameName .. "Server")
	local teamIconObj = getglobal(teamFrameName .. "TeamIcon")
	local numPlayersObj = getglobal(teamFrameName .. "NumPlayers")
	local battleScoreObj = getglobal(teamFrameName .. "BattleScore")
	local ratingObj = getglobal(teamFrameName .. "Rating")
	local ratingChangeObj = getglobal(teamFrameName .. "RatingChange")

	if (team.teamId == WarRecord.TEAM_ID_BLUE) then
		backgroundBarObj:SetColor(0, 0, 1)
		teamIconObj:SetTexture("Interface/GuildHouseFrame/GuildHouseWar-GuildName2")  -- out of order
	elseif (team.teamId == WarRecord.TEAM_ID_RED) then
		backgroundBarObj:SetColor(1, 0, 0)
		teamIconObj:SetTexture("Interface/GuildHouseFrame/GuildHouseWar-GuildName1")  -- out of order
	else
		backgroundBarObj:SetColor(1, 1, 1)
		teamIconObj:SetTexture("")
	end

	if (team.guildName) and (team.guildServer) then
		nameObj:SetText(team.guildName)
		serverObj:SetText(string.format(WarRecord.Locale.FORMAT_SERVER, team.guildServer))
	else
		nameObj:SetText(WarRecord.Locale.UNKNOWN_STR)
		serverObj:SetText(string.format(WarRecord.Locale.FORMAT_SERVER, WarRecord.Locale.UNKNOWN_STR))
	end

	if (team.participants) then
		if (team.participants == 1) then
			numPlayersObj:SetText(string.format(WarRecord.Locale.FORMAT_PARTICIPANT_SINGLE, team.participants))
		else
			numPlayersObj:SetText(string.format(WarRecord.Locale.FORMAT_PARTICIPANTS_PLURAL, team.participants))
		end
	else
		numPlayersObj:SetText(WarRecord.Locale.UNKNOWN_STR)
	end

	if (team.battleScore) then
		battleScoreObj:SetText(string.format(WarRecord.Locale.FORMAT_BATTLE_SCORE, team.battleScore))
	else
		battleScoreObj:SetText(WarRecord.Locale.UNKNOWN_STR)
	end

	if (team.startRating) then
		ratingObj:SetText(string.format(WarRecord.Locale.FORMAT_RATING, team.startRating))
	else
		ratingObj:SetText(string.format(WarRecord.Locale.FORMAT_RATING, WarRecord.Locale.UNKNOWN_STR))
	end

	local ratingChange = nil
	if (team.endRating) and (team.startRating) then
		ratingChange = team.endRating - team.startRating
	end
	if (not ratingChange) then
		ratingChangeObj:SetText(WarRecord.Locale.UNKNOWN_RATING_CHANGE)
	elseif (ratingChange == 0) then
		ratingChangeObj:SetText(WarRecord.Locale.RATING_NO_CHANGE)
	else
		ratingChangeObj:SetText(string.format(WarRecord.Locale.FORMAT_RATING_CHANGE, ratingChange))
	end
end

--------------------------------------------------------------------------------

function WarRecord.ScoreBoardSort()
	table.sort(displayFilteredPlayers, currentScoreBoardSortFunction)

	WarRecord.FilteredDisplayFindSelfPlayer()

	WarRecordStatusFrameListFrame:Update()
	WarRecordStatusFrameRewardFrame:Update()
end

--------------------------------------------------------------------------------

function WarRecord.GuildHouseWarStatusFrameTab_OnClick(tab)
	PanelTemplates_SetTab(WarRecordStatusFrame, tab);

	if (tab == WarRecord.SCOREBOARD_TAB_US) then
		WarRecord.FilteredDisplayCreate(displayData.usId)
	elseif (tab == WarRecord.SCOREBOARD_TAB_ENEMY) then
		WarRecord.FilteredDisplayCreate(displayData.enemyId)
	else
		WarRecord.FilteredDisplayCreate()
	end
	WarRecord.ScoreBoardSort()
end


function WarRecord.GuildHouseWarStatusFrameColumnButton_OnClick(frame)
	local buttonId = frame:GetID()
	local newSort = currentScoreBoardSortFunction

	-- Find next sort function to use based on column.
	local i, v
	for i, v in ipairs(sortScoreBoard) do
		if (scorePage == v.page) and (buttonId == v.id) then
			if (currentScoreBoardSortFunction == v.compFunc1) then
				newSort = v.compFunc2
			else
				newSort = v.compFunc1
			end
			break
		end
	end

	if (currentScoreBoardSortFunction ~= newSort) then
		currentScoreBoardSortFunction = newSort
		WarRecord.ScoreBoardSort()
	end
end

--------------------------------------------------------------------------------

-- Set up UIPanelCommonListFrame scrollbar for init.
function WarRecord.ListScrollInit(listFrame, info)
	info.scrollBarType = 3
	info.scrollBarStatic = 1
	-- Disable scroll buttons initially.  Update will enable them if the range is greater than 0.
	getglobal(listFrame:GetName().."ScrollBarScrollUpButton"):Disable()
	getglobal(listFrame:GetName().."ScrollBarScrollDownButton"):Disable()
end

--------------------------------------------------------------------------------

function WarRecord.GuildHouseWarStatusFrame_OnLoad(frame)
	UIPanelBackdropFrame_SetTexture(frame, "Interface/Common/PanelCommonFrame", 256, 256)
	getglobal(frame:GetName() .. "Title"):SetText(BG_STATUS_FRAME_TITLE)

	PanelTemplates_SetNumTabs(frame, MAX_GHS_STATUS_FRAME_TAB_NUM)
	PanelTemplates_SetTab(frame, WarRecord.SCOREBOARD_TAB_MIX)

	local info = {}
	info.columnTemplate = "WarRecordStatusColumnFrameTemplate"
	info.itemTemplate = "WarRecordStatusItemTemplate"
	info.messageBarTemplate = "WarRecordStatusMessageBarTemplate"
	info.itemCount = MAX_SCOREBOARD_LIST_ITEMS
	info.callbackFunc_GetCount = WarRecord.GHS_ListFrame_CallbackFunc_GetCount
	info.callbackFunc_UpdateItme = WarRecord.GHS_ListFrame_CallbackFunc_UpdateItem
	WarRecord.ListScrollInit(WarRecordStatusFrameListFrame, info)
	UIPanelCommonListFrame_Init(WarRecordStatusFrameListFrame, info)

	info.columnTemplate = "WarRecordRewardColumnFrameTemplate"
	info.itemTemplate = "WarRecordRewardItemTemplate"
	info.messageBarTemplate = "WarRecordRewardMessageBarTemplate"
	info.itemCount = MAX_SCOREBOARD_LIST_ITEMS
	info.callbackFunc_GetCount = WarRecord.GHS_RewardFrame_CallbackFunc_GetCount
	info.callbackFunc_UpdateItme = WarRecord.GHS_RewardFrame_CallbackFunc_UpdateItem
	WarRecord.ListScrollInit(WarRecordStatusFrameRewardFrame, info)
	UIPanelCommonListFrame_Init(WarRecordStatusFrameRewardFrame, info)
end

--------------------------------------------------------------------------------

function WarRecord.GHS_ListFrame_CallbackFunc_GetCount(listFrame)
	if (displayFilteredPlayers.selfPlayerIndex ~= nil) then
		listFrame.messageBar:Show()
		WarRecord.GHS_ListFrame_CallbackFunc_UpdateItem(listFrame, listFrame.messageBar, displayFilteredPlayers.selfPlayerIndex)
	else
		listFrame.messageBar:Hide()
	end

	return #(displayFilteredPlayers)
end


function WarRecord.GHS_ListFrame_CallbackFunc_UpdateItem(listFrame, item, index)
	local itemObj = item
	local NameObj = getglobal(itemObj:GetName() .. "ColumnName")
	local KillObj = getglobal(itemObj:GetName() .. "ColumnKill")
	local DmgObj = getglobal(itemObj:GetName() .. "ColumnDmg")
	local DfdObj = getglobal(itemObj:GetName() .. "ColumnDfd")
	local HealObj = getglobal(itemObj:GetName() .. "ColumnHeal")
	local MeritObj = getglobal(itemObj:GetName() .. "ColumnMerit")
	local EnergyObj = getglobal(itemObj:GetName() .. "ColumnEnergy")
	local OccupyObj = getglobal(itemObj:GetName() .. "ColumnOccupy")
	local TextureObj = getglobal(itemObj:GetName() .. "Texture")

	NameObj:SetText(displayFilteredPlayers[index].name)
	KillObj:SetText(displayFilteredPlayers[index].kills)
	DmgObj:SetText(displayFilteredPlayers[index].damage)
	DfdObj:SetText(displayFilteredPlayers[index].defend)
	HealObj:SetText(displayFilteredPlayers[index].healing)
	MeritObj:SetText(displayFilteredPlayers[index].merits)
	EnergyObj:SetText(displayFilteredPlayers[index].energy)
	OccupyObj:SetText(displayFilteredPlayers[index].occupations)

	if (displayFilteredPlayers[index].teamId == WarRecord.TEAM_ID_BLUE) then
		TextureObj:SetColor(0, 0, 1)
	elseif (displayFilteredPlayers[index].teamId == WarRecord.TEAM_ID_RED) then
		TextureObj:SetColor(1, 0, 0)
	end
end

--------------------------------------------------------------------------------

function WarRecord.GHS_RewardFrame_CallbackFunc_GetCount(listFrame)
	if (displayFilteredPlayers.selfPlayerIndex ~= nil) then
		listFrame.messageBar:Show()
		WarRecord.GHS_RewardFrame_CallbackFunc_UpdateItem(listFrame, listFrame.messageBar, displayFilteredPlayers.selfPlayerIndex)
	else
		listFrame.messageBar:Hide()
	end

	return #(displayFilteredPlayers)
end


function WarRecord.GHS_RewardFrame_CallbackFunc_UpdateItem(listFrame, item, index)
	item:SetID(index)

	local itemObj = item
	local NameObj = getglobal(itemObj:GetName() .. "ColumnName")
	local VocObj = getglobal(itemObj:GetName() .. "ClassIconTexture")
	local LvObj = getglobal(itemObj:GetName() .. "ClassIconNum")
	local SubVocObj = getglobal(itemObj:GetName() .. "SubClassIconTexture")
	local SubLvObj = getglobal(itemObj:GetName() .. "SubClassIconNum")
	local RewardTypeObj = getglobal(itemObj:GetName() .. "ColumnTypeText")
	local ScoreObj = getglobal(itemObj:GetName() .. "ColumnScore")
	local RankObj = getglobal(itemObj:GetName() .. "ColumnRank")
	local TextureObj = getglobal(itemObj:GetName() .. "Texture")

	NameObj:SetText(displayFilteredPlayers[index].name)

	if (displayFilteredPlayers[index].class) then
		GuildHouseWarStatusFrame_SetClassIcon(VocObj, displayFilteredPlayers[index].class.primary)
		LvObj:SetText(displayFilteredPlayers[index].class.primeLevel)
		GuildHouseWarStatusFrame_SetClassIcon(SubVocObj, displayFilteredPlayers[index].class.secondary)
		if (displayFilteredPlayers[index].class.secondLevel > 0) then
			SubLvObj:SetText(displayFilteredPlayers[index].class.secondLevel)
		else
			SubLvObj:SetText("")
		end
	end

	local text = "GHS_REWARD_TYPE" .. (displayFilteredPlayers[index].unitType)
	RewardTypeObj:SetText(TEXT(text))

	if (displayFilteredPlayers[index].score ~= -1) then
		ScoreObj:SetText(displayFilteredPlayers[index].score)
	else
		ScoreObj:SetText(DASH .. DASH)
	end

	if (displayFilteredPlayers[index].rank ~= -1) then
		RankObj:SetText(displayFilteredPlayers[index].rank)
	else
		RankObj:SetText(DASH .. DASH)
	end

	if (displayFilteredPlayers[index].teamId == WarRecord.TEAM_ID_BLUE) then
		TextureObj:SetColor(0, 0, 1)
	elseif (displayFilteredPlayers[index].teamId == WarRecord.TEAM_ID_RED) then
		TextureObj:SetColor(1, 0, 0)
	end
end

--------------------------------------------------------------------------------

function WarRecord.GuildHouseWarStatusFrame_OnShow(frame)
	frame:ResetFrameOrder()

	WarRecordStatusFrameListFrame:Show()
	WarRecordStatusFrameRewardFrame:Hide()
	WarRecordStatusFrameWarReward:SetText(GHS_WAR_REWARD)
	scorePage = WarRecord.SCORE_PAGE_STATUS

	WarRecordStatusFrameListFrame:Update()
	WarRecordStatusFrameRewardFrame:Update()
end

--------------------------------------------------------------------------------

function WarRecord.GuildHouseWarRewardItemClass_OnEnter(frame)
	if (displayFilteredPlayers == nil) then
		return
	end

	local parent = frame:GetParent()
	local parentId  = parent:GetID()
	local itemData = displayFilteredPlayers[parentId]
	local id = frame:GetID()
	local class = -1
	local lv = 0

	if (itemData == nil) then
		return
	end

	if (id == 1) then
		class = itemData.class.primary
		lv = itemData.class.primeLevel
	elseif (id == 2) then
		class = itemData.class.secondary
		lv = itemData.class.secondLevel
	end

	if (class > -1) then
		GameTooltip:SetOwner(frame:GetName(), "ANCHOR_RIGHT", 4, 24)
		local text = string.format("%s LV%d", GetBattleGroundPlayerClassName(class), lv)

		GameTooltip:SetText(text, 1, 1, 1)
		GameTooltip:Show()
	end
end


function WarRecord.GuildHouseWarRewardItemClass_OnLeave(frame)
	GameTooltip:Hide()
end

--------------------------------------------------------------------------------

function WarRecord.GuildHouseWarRewardItemColumnType_OnEnter(frame)
	if (displayFilteredPlayers == nil) then
		return
	end

	local parent = frame:GetParent()
	local parentId = parent:GetID()
	local itemData = displayFilteredPlayers[parentId]

	if (itemData == nil or itemData.unitType < 1) then
		return
	end

	local text = TEXT("GHS_REWARD_TYPE" .. (itemData.unitType) .. "_TIPS")

	GameTooltip:SetOwner(frame:GetName(), "ANCHOR_RIGHT", 4, 24)
	GameTooltip:SetText(text, 1, 1, 1)
	GameTooltip:Show()
end


function WarRecord.GuildHouseWarRewardItemColumnType_OnLeave(frame)
	GameTooltip:Hide()
end

--------------------------------------------------------------------------------

function WarRecord.GuildHouseWarStatusFrameWarReward_OnClick(frame)
	if (WarRecordStatusFrameListFrame:IsVisible()) then
		WarRecordStatusFrameListFrame:Hide()
		WarRecordStatusFrameRewardFrame:Show()
		WarRecordStatusFrameWarReward:SetText(BG_STATUS_FRAME_TITLE)

		scorePage = WarRecord.SCORE_PAGE_REWARD
	else
		WarRecordStatusFrameListFrame:Show()
		WarRecordStatusFrameRewardFrame:Hide()
		WarRecordStatusFrameWarReward:SetText(GHS_WAR_REWARD)

		scorePage = WarRecord.SCORE_PAGE_STATUS
	end
end
