
local currentRecord = {
	teamRecord = {},
	timestamps = {}
}

local waitingForFightEnd = false
local currentRecordState = nil
local currentRecordGuild = nil
local firstTeamCheck = true

local DEFERRED_TIME_STATE_CHANGE = 8
local DEFERRED_TIME_SEEK_STATE_CHANGE = 2
local DEFERRED_TIME_SEEK_FIGHT_END = 60
local LIMIT_TIME_SEEK_STATE_CHANGE = 45
local LIMIT_TIME_SEEK_FIGHT_END = 3720

WarRecord.Locale.ERROR_TIMEOUT_FIGHT_END = WarRecord.Locale.ADDON_NAME.." fault:  siege end-of-battle condition expected, but no notification received."

--------------------------------------------------------------------------------

function WarRecord.OnGuildHouseClose()
	currentRecord.timestamps.battleClose = WarRecord.CurrentTimeStamp()
	WarRecord.TeamRecordsClose(currentRecord)
	WarRecord.RecordAllPlayerScores(currentRecord)
	waitingForFightEnd = false
	WarRecord.PrepareDeferredProc(DEFERRED_TIME_STATE_CHANGE, WarRecord.RequestRegistrationInfo)
	WarRecord.CountParticipants(currentRecord)
end


function WarRecord.GhwInfosUpdate()
	if (GetGuildInfo() ~= currentRecordGuild) then
		firstTeamCheck = true
		currentRecordGuild = GetGuildInfo()
	end

	local _, state
	local selfGuildIndex = WarRecord.FindGuildRegistration()
	if not (selfGuildIndex) then
		state = nil
	else
		_,_,_,_,_,_,_,state = GuildHousesWar_GetRegisterInfo(selfGuildIndex)
	end

	if (state ~= currentRecordState) then
		-- State has changed.
		currentRecordState = state
		WarRecord.lookingForStateChange = false
		waitingForFightEnd = false
		WarRecord.UpdateTeams()
	else
		-- Expecting a state change soon.  Queue another check in a few seconds.
		if (WarRecord.lookingForStateChange) then
			if (GetTime() - WarRecord.lastStateChangeTime < LIMIT_TIME_SEEK_STATE_CHANGE) then
				WarRecord.PrepareDeferredProc(DEFERRED_TIME_SEEK_STATE_CHANGE, WarRecord.RequestRegistrationInfo)
			else
				WarRecord.lookingForStateChange = false
			end
		elseif (waitingForFightEnd) then
			if (GetTime() - WarRecord.lastStateChangeTime < LIMIT_TIME_SEEK_FIGHT_END) then
				WarRecord.PrepareDeferredProc(DEFERRED_TIME_SEEK_FIGHT_END, WarRecord.RequestRegistrationInfo)
			else
				waitingForFightEnd = false
				WarRecord.OutputText(WarRecord.Locale.ERROR_TIMEOUT_FIGHT_END, 0.5, 0.5, 0.5)
			end
		end
	end
end

--------------------------------------------------------------------------------

-- true if the last stored battle matches the current information
function WarRecord.CompareFightEnd(blueName, blueServer, blueScore, redName, redServer, redScore)
	if (#(WarRecordProfile.BattleHistory) == 0) then
		return false
	end
	local lastRecord = WarRecordProfile.BattleHistory[#(WarRecordProfile.BattleHistory)]

	-- Assume only records with FightEnd (endRating) data are stored.
	if (lastRecord.teamRecord[1].guildName == blueName) and
		(lastRecord.teamRecord[1].guildServer == blueServer) and
		(lastRecord.teamRecord[1].endRating == blueScore) and
		(lastRecord.teamRecord[2].guildName == redName) and
		(lastRecord.teamRecord[2].guildServer == redServer) and
		(lastRecord.teamRecord[2].endRating == redScore) then
		return true
	elseif (lastRecord.teamRecord[2].guildName == blueName) and
		(lastRecord.teamRecord[2].guildServer == blueServer) and
		(lastRecord.teamRecord[2].endRating == blueScore) and
		(lastRecord.teamRecord[1].guildName == redName) and
		(lastRecord.teamRecord[1].guildServer == redServer) and
		(lastRecord.teamRecord[1].endRating == redScore) then
		return true
	end
	return false
end


function WarRecord.AppendBattle()
	currentRecord.timestamps.recorded = WarRecord.CurrentTimeStamp()
	-- discard empty table
	if (not next(currentRecord.timestamps)) then
		currentRecord.timestamps = nil
	end

	table.insert(WarRecordProfile.BattleHistory, currentRecord)

	-- new current record
	currentRecord = {
		teamRecord = {},
		timestamps = {}
	}

	WarRecord.TrimBattleHistory()
	WarRecord.BattleList_CreateSortableHistory()
end


function WarRecord.IdentifySelfTeam()
	local selfGuildName = GetGuildInfo()
	local selfServerName = GetServerName()

	local i
	for i = 1, #(currentRecord.teamRecord) do
		if (currentRecord.teamRecord[i].guildName == selfGuildName) and
			(currentRecord.teamRecord[i].guildServer == selfServerName) then
			currentRecord.selfTeamIndex = i
		end
	end
end

--------------------------------------------------------------------------------

function WarRecord.UpdateTeams()
	if (firstTeamCheck) then
		WarRecord.UpdateTeamsInitial()
		firstTeamCheck = false
	else
		WarRecord.UpdateTeamsStateChange()
	end
end


function WarRecord.UpdateTeamsInitial()
	-- if fight-end state, and teams fully match (unordered) info, do nothing
	-- if fight-end state, and mismatch, create new record and append (currentRecord cleared upon fight-end state after appending)
	-- else, fill in currentRecord.

	local selfGuildIndex = WarRecord.FindGuildRegistration()
	if not (selfGuildIndex) then
		return
	end
	local blueName, blueServer, blueScore, redName, redServer, redScore, time, state = GuildHousesWar_GetRegisterInfo(selfGuildIndex)

	if (state == "FightEnd_Even") or (state == "FightEnd_Win") or (state == "FightEnd_Lost") then
		-- if unordered teams match name/server/score, do nothing
		-- else, fill in currentRecord and append
		if not (WarRecord.CompareFightEnd(blueName, blueServer, blueScore, redName, redServer, redScore)) then
			currentRecord.teamRecord[1] = {
				guildName = blueName,
				guildServer = blueServer,
				endRating = blueScore
			}
			currentRecord.teamRecord[2] = {
				guildName = redName,
				guildServer = redServer,
				endRating = redScore
			}
			WarRecord.IdentifySelfTeam()
			currentRecord.victorIndex = WarRecord.StateVictor(state, {1, 2})
			WarRecord.AppendBattle()
		end
	elseif (state == "Prepare") or (state == "Fight") then
		-- both teams valid, and in color order
		currentRecord.teamRecord[1] = {
			guildName = blueName,
			guildServer = blueServer,
			startRating = blueScore,
			teamId = WarRecord.TEAM_ID_BLUE
		}
		currentRecord.teamRecord[2] = {
			guildName = redName,
			guildServer = redServer,
			startRating = redScore,
			teamId = WarRecord.TEAM_ID_RED
		}
		WarRecord.IdentifySelfTeam()
		if (state == "Fight") then
			waitingForFightEnd = true
			WarRecord.PrepareDeferredProc(DEFERRED_TIME_SEEK_FIGHT_END, WarRecord.RequestRegistrationInfo)
		end
	end
end


function WarRecord.UpdateTeamsStateChange()
	-- if new state == prepare/fight, fill in currentRecord.
	-- if new state == fight-end, match teams up, update ratings, append to battle list, and clear currentRecord.
	-- else, currentRecord should already be cleared.

	local selfGuildIndex = WarRecord.FindGuildRegistration()
	if not (selfGuildIndex) then
		return
	end
	local blueName, blueServer, blueScore, redName, redServer, redScore, time, state = GuildHousesWar_GetRegisterInfo(selfGuildIndex)

	if (state == "Prepare") or (state == "Fight") then
		-- both teams valid, and in color order
		currentRecord.teamRecord[1] = {
			guildName = blueName,
			guildServer = blueServer,
			startRating = blueScore,
			teamId = WarRecord.TEAM_ID_BLUE
		}
		currentRecord.teamRecord[2] = {
			guildName = redName,
			guildServer = redServer,
			startRating = redScore,
			teamId = WarRecord.TEAM_ID_RED
		}
		WarRecord.IdentifySelfTeam()
		if (state == "Fight") then
			currentRecord.timestamps.battleStart = WarRecord.CurrentTimeStamp()
			waitingForFightEnd = true
			WarRecord.PrepareDeferredProc(DEFERRED_TIME_SEEK_FIGHT_END, WarRecord.RequestRegistrationInfo)
		end
	elseif (state == "FightEnd_Even") or (state == "FightEnd_Win") or (state == "FightEnd_Lost") then
		-- Swap order if necessary.
		local idToIndex
		if (currentRecord.teamRecord[1].guildName == blueName) and
			(currentRecord.teamRecord[1].guildServer == blueServer) and
			(currentRecord.teamRecord[2].guildName == redName) and
			(currentRecord.teamRecord[2].guildServer == redServer) then
			idToIndex = {1, 2}
		elseif (currentRecord.teamRecord[1].guildName == redName) and
			(currentRecord.teamRecord[1].guildServer == redServer) and
			(currentRecord.teamRecord[2].guildName == blueName) and
			(currentRecord.teamRecord[2].guildServer == blueServer) then
			idToIndex = {2, 1}
		else
			return
		end
		currentRecord.teamRecord[idToIndex[WarRecord.TEAM_ID_BLUE]].endRating = blueScore
		currentRecord.teamRecord[idToIndex[WarRecord.TEAM_ID_RED]].endRating = redScore
		if not (currentRecord.victorIndex) then
			currentRecord.victorIndex = WarRecord.StateVictor(state, idToIndex)
		end
		WarRecord.AppendBattle()
	end
end

--------------------------------------------------------------------------------

function WarRecord.StateVictor(state, idToIndex)
	if (state == "FightEnd_Win") then
		return idToIndex[WarRecord.TEAM_ID_BLUE]
	elseif (state == "FightEnd_Lost") then
		return idToIndex[WarRecord.TEAM_ID_RED]
	else --(state == "FightEnd_Even")
		return 0
	end
end

--------------------------------------------------------------------------------

function WarRecord.FightEndProbe()
	if (not GuildHousesWar_IsInBattleGround()) then
		local selfGuildIndex = WarRecord.FindGuildRegistration()
		if not (selfGuildIndex) then
			return false
		end
		local blueName, blueServer, blueScore, redName, redServer, redScore, time, state = GuildHousesWar_GetRegisterInfo(selfGuildIndex)

		if (state == "FightEnd_Even") or (state == "FightEnd_Win") or (state == "FightEnd_Lost") then
			return true
		end
	end
	return false
end


