
-- Namespace for this add-on
WarRecord = {}
WarRecord.Locale = {}

-- Constant text
WarRecord.VERSION_STR = "v1.1.1"
WarRecord.AUTHOR = "Jaamosan"

-- Localizable text
WarRecord.Locale.ADDON_NAME = "War Record"
WarRecord.Locale.DESCRIPTION = "Record of battles past"

WarRecord.Locale.FORMAT_ERROR_PLAYER_QUERY_MISMATCH = WarRecord.Locale.ADDON_NAME.." error:  query mismatch.  %s (team %s) ~= %s (team %s)"


WarRecord.TEAM_ID_BLUE = 1
WarRecord.TEAM_ID_RED = 2  -- BG_STATUS_FRAME_TEAM1 (out of order)

WarRecord.SCOREBOARD_TAB_US = 1
WarRecord.SCOREBOARD_TAB_ENEMY = 2
WarRecord.SCOREBOARD_TAB_MIX = 3

WarRecord.SCORE_PAGE_STATUS = 1
WarRecord.SCORE_PAGE_REWARD = 2


-- Variables that will be stored
WarRecordSaves = {}
WarRecordProfile = {
	BattleHistory = {}
}

local defaultConfig = {
	maxEntries = 90,
	maxEntriesUnlimited = true,
	maxDetailedEntries = 30,
	maxDetailedEntriesUnlimited = true,
	serverTimeOffset = 0,
	timestampFormat = "%d/%m"
}


WarRecord.lookingForStateChange = false
WarRecord.lastStateChangeTime = nil

-- Defer operation until data is available.
local WaitingCalls = {}

--------------------------------------------------------------------------------

function WarRecord.OutputText(msg, red, green, blue)
	DEFAULT_CHAT_FRAME:AddMessage(msg, red, green, blue)
end

--------------------------------------------------------------------------------

function WarRecord.Frame_OnLoad(frame)
	frame:RegisterEvent("VARIABLES_LOADED")
	frame:RegisterEvent("ON_GUILDHOUSE_CLOSE")
	frame:RegisterEvent("GUILDHOUSEWAR_INFOS_UPDATE")
	frame:RegisterEvent("GUILDHOUSEWAR_STATE_CHANGE")

	SaveVariables("WarRecordSaves")
	SaveVariablesPerCharacter("WarRecordProfile")
end


function WarRecord.VariablesLoaded()
	local k, v

	-- Locale default.  (UK, AU use "ENEU".)
	if (GetLanguage():upper() == "ENUS") then
		defaultConfig.timestampFormat = "%m/%d"
	end

	-- if no saved values, copy from defaults
	for k, v in pairs(defaultConfig) do
		if (WarRecordSaves[k] == nil) then
			WarRecordSaves[k] = v
		end
	end

	WarRecord.BattleList_CreateSortableHistory()

	WarRecord.lastStateChangeTime = GetTime()
	WarRecord.RequestRegistrationInfo()

	-- registration with optional AddonManager
	WarRecord.AddonManagerRegister()
end


function WarRecord.Frame_OnEvent(frame, event)
	if (event == "VARIABLES_LOADED") then
		WarRecord.VariablesLoaded()
	elseif (event == "ON_GUILDHOUSE_CLOSE") then
		WarRecord.OnGuildHouseClose()
	elseif (event == "GUILDHOUSEWAR_STATE_CHANGE") then
		WarRecord.lastStateChangeTime = GetTime()
		WarRecord.lookingForStateChange = true
		WarRecord.RequestRegistrationInfo()
	elseif (event == "GUILDHOUSEWAR_INFOS_UPDATE") then
		WarRecord.GhwInfosUpdate()
	end
end


function WarRecord.Frame_OnUpdate(frame, elapsedTime)
	local i = 1
	while (WaitingCalls[i]) do
		-- Tend deferred procedures and execute any that are ready.
		WaitingCalls[i].timeRemaining = WaitingCalls[i].timeRemaining - elapsedTime
		if (WaitingCalls[i].timeRemaining < 0) then
			WaitingCalls[i].callback()
			table.remove(WaitingCalls, i)
		else
			i = i + 1
		end
	end
end

--------------------------------------------------------------------------------

-- Set up a procedure to be called after some time has elapsed.
function WarRecord.PrepareDeferredProc(time, proc)
	-- if proc is already queued, just reset the time
	local k,v
	for k, v in pairs(WaitingCalls) do
		if (v.callback == proc) then
			v.timeRemaining = time
			return
		end
	end

	table.insert(WaitingCalls, {timeRemaining = time, callback = proc})
end

--------------------------------------------------------------------------------

-- update internal state that queries receive data from
function WarRecord.RequestRegistrationInfo()
	GuildHousesWar_OpenMenu()
end

--------------------------------------------------------------------------------

-- return index into registration table
function WarRecord.FindGuildRegistration()
	local selfGuildName = GetGuildInfo()
	local selfServerName = GetServerName()

	local count = GuildHousesWar_GetRegisterCount()
	local i

	if not (selfGuildName) then
		return nil
	end

	for i = 1, count do
		local blueName, blueServer, blueScore, redName, redServer, redScore, time, state = GuildHousesWar_GetRegisterInfo(i)

		if ((selfGuildName == blueName and selfServerName == blueServer) or 
			(selfGuildName == redName and selfServerName == redServer)) then
			return i
		end
	end
	return nil
end

--------------------------------------------------------------------------------

function WarRecord.TeamRecordsClose(battle)
	local blueEnergy, redEnergy, blueScore, redScore = GetGuildHouseWarInfo()
	battle.teamRecord[1].battleScore = blueScore
	battle.teamRecord[2].battleScore = redScore

	battle.victorIndex = GetGuildHouseWarWinnerTeam()
	-- victorIndex could also be found by looking at the aftermath state.
end

--------------------------------------------------------------------------------

function WarRecord.CountParticipants(battle)
	for k, v in pairs(battle.teamRecord) do
		v.participants = #(v.playerRecord)
	end
end


function WarRecord.RecordAllPlayerScores(battle)
	local i

	GuildHouseWarStatusFrameTabChange(WarRecord.SCOREBOARD_TAB_MIX)

	local numPlayers = GetGuildHouseWarPlayerScoreInfoNum()

	battle.teamRecord[1].playerRecord = {}
	battle.teamRecord[2].playerRecord = {}
	local record, teamId
	for i = 1, numPlayers do
		record, teamId = WarRecord.GetPlayerRecord(i)
		table.insert(battle.teamRecord[teamId].playerRecord, record)
	end

	GuildHouseWarStatusFrameTabChange(WarRecord.SCOREBOARD_TAB_US)
end


function WarRecord.GetPlayerRecord(index)
	local record = {class = {}}

	local iIsPlayer, iTeamColor, Name, iKillVal, iDamageVal, iDefendVal, iHealVal, iMeritVal, iEnergyVal, iOccupyVal = GetGuildHouseWarPlayerScoreInfo(index - 1, WarRecord.SCORE_PAGE_STATUS)

	if (iIsPlayer == 1) then
		record.iIsPlayer = 1
	end
	record.name = Name
	record.kills = iKillVal
	record.damage = iDamageVal
	record.defend = iDefendVal
	record.healing = iHealVal
	record.merits = iMeritVal
	record.energy = iEnergyVal
	record.occupations = iOccupyVal

	local iIsPlayer2, iTeamColor2, Name2, Voc, Lv, subVoc, subLv, iRewardType, iScoreVal, iRankVal = GetGuildHouseWarPlayerScoreInfo(index - 1, WarRecord.SCORE_PAGE_REWARD)
	if (iTeamColor ~= iTeamColor2) or (Name ~= Name2) then
		WarRecord.OutputText(string.format(WarRecord.Locale.FORMAT_ERROR_PLAYER_QUERY_MISMATCH, tostring(Name), tostring(iTeamColor), tostring(Name2), tostring(iTeamColor2)), 0.5, 0.5, 0.5)
	end

	record.class.primary = Voc
	record.class.primeLevel = Lv
	record.class.secondary = subVoc
	record.class.secondLevel = subLv
	record.unitType = iRewardType + 1
	record.rank = iRankVal
	record.score = iScoreVal

	return record, iTeamColor
end

--------------------------------------------------------------------------------

function WarRecord.TrimBattleHistory()
	if (not WarRecordSaves.maxEntriesUnlimited) then
		local trimEntries = #(WarRecordProfile.BattleHistory) - WarRecordSaves.maxEntries

		while (trimEntries > 0) do
			table.remove(WarRecordProfile.BattleHistory, 1)
			trimEntries = trimEntries - 1
		end
	end

	if (not WarRecordSaves.maxDetailedEntriesUnlimited) then
		local i, k, v

		for i = 1, (#(WarRecordProfile.BattleHistory) - WarRecordSaves.maxDetailedEntries) do
			for k, v in pairs(WarRecordProfile.BattleHistory[i].teamRecord) do
				v.playerRecord = nil
			end
		end
	end
end

