-----------------------------------------------------------------------------
--  DailyNotes is an Addon for Runes of Magic
--  It will show you additional information in tooltips and on bulleting boards.
--  feedback to: viertelvor12@gmx.net
--
--  License: MIT/X
-----------------------------------------------------------------------------

------------------------------
--- another ROM-Bug workaround
------------------------------
local myGetZoneLocalName = GetZoneLocalName
function GetZoneLocalName(id)
 if (id==-1) then return TEXT("GUILD_TEXT_ZONE_UNKOWN") end
 return myGetZoneLocalName(id)
end
------------------------------
local Nyx = LibStub("Nyx")
------------------------------

local DailyNotes = {

  MAX_LIST_ENTRIES = 15, -- list entries in dialog
  version       = "v4.0.4",


  -- runtime vars:
  Quest_count = 0,    -- total count of quests in DB
  DB_Mobs = nil,      -- maps: mob names to quest ids
  Last_daily_count= 0,-- how much quest has been done
}
_G.DailyNotes = DailyNotes

DN_F_TITLE = "DailyNotes  |cffa0a000"..DailyNotes.version


-------------------------------------
function DailyNotes.Config_Reset()

    DN_Options={
        version = DailyNotes.version,

        -- DEFAULT OPTIONS:
        showcharlevel = true,   -- Show required level instead of quest level
        takefilter = true,      -- Only auto-take quest if items are in inventory
        verbose = false,        -- no additional infos
        verbose_quest = true,   -- no quest infos
        itemcount = true,       -- show item count in loot
         dqionly = true,        -- ^ but only for DQ items

        TT_mobs  = true,        -- tool tip on mobs
        TT_items = true,        -- tool tip on items
        TT_dialog= true,        -- tool tip in dialog
        ZBag_Hook= true,        -- apply bag highlights in ZBag
    }

    local realm = Nyx.GetCurrentRealm()
    if realm~="" then
        DN_Options_Of_Char[realm]={}
        DN_Char_Options = DN_Options_Of_Char[realm]
    end
end

-- init Options
DN_Options={}
DN_Options_Of_Char={}
local DN_Char_Options={}
DailyNotes.Config_Reset()


-----------------------------------------------------------------------
-- Database loading
-----------------------------------------------------------------------
function DailyNotes.LoadDBandLocales()
    Nyx.LoadLocalesAuto("Interface/Addons/dailynotes/Locales/")
    -- Nyx.LoadLocales("Interface/Addons/dailynotes/Locales/","tw")

    Nyx.LoadFile("Interface/Addons/dailynotes/quest_db.lua")

    DailyNotes.Quest_count = Nyx.TableSize(DailyNotes.DB_Quests)
end

-- LOAD DB
DailyNotes.LoadDBandLocales()


function DailyNotes.FindQuestForItem(itemID)

    local q,qdata

    for QID,e in pairs(DailyNotes.DB_Quests) do
        if e.items and e.items[itemID] then
            if CheckQuest(QID)==1 then
                return QID,e
            end
            q=QID
            qdata=e
        end
    end

    return q,qdata
end

function DailyNotes.IsQuestGiver(NPCid)

    for _,e in pairs(DailyNotes.DB_Quests) do

        if type(e.giver)=="number" then
            if e.giver==NPCid then
                return true
            end
        else
            for _,nid in ipairs(e.giver) do
                if nid==NPCid then return true end
            end
        end
    end
end


-- TODO: this is a quickworkaround .. the speakframe_changed it self need some rework
local EXTRA_TEXT = "["..UI_QUEST_MSG_DAILY.."]"

local function IsNpcAQuestGiverOrTaker(npc_name, qdata)
    if type(qdata.giver)=="number" then
        return npc_name==TEXT("Sys"..qdata.giver.."_name")
    else
        for _,nid in ipairs(qdata.giver) do
            if npc_name==TEXT("Sys"..nid.."_name") then return true end
        end
    end

    if qdata.taker and npc_name==TEXT("Sys"..qdata.taker.."_name")then
        return true
    end

end

function DailyNotes.FindQuestByName(qname, npc)

    local found_quest
    for QID,e in pairs(DailyNotes.DB_Quests) do
        if TEXT("Sys"..QID.."_name")..EXTRA_TEXT==qname or TEXT("Sys"..QID.."_name")==qname  then
            if not npc or IsNpcAQuestGiverOrTaker(npc,e) then
                return QID,e
            end

            found_quest = QID
        end
    end

    if found_quest and DN_Options.debug then
        DEFAULT_CHAT_FRAME:AddMessage("|cffff0000DN: Quest "..found_quest.." not for found for NPC "..npc)
    end
end

function DailyNotes.GetQuestItems(qudata)

  if not qudata then return end
  if not qudata.items then return 0,{},0 end

  local itemcounts = {}
  local max_completed=nil
  local finish_able
  local total=0
  for iid,count in pairs(qudata.items) do
    itemcounts[iid] = Nyx.GetTotalBagItemCount(iid)
    finish_able = math.floor(itemcounts[iid]/count)
    total = total + itemcounts[iid]
    if not max_completed or finish_able<max_completed then
      max_completed = finish_able
    end
  end

  return max_completed,itemcounts,total
end

------------------------------
-- return: Name,map,pos_x,pos_y,alt_map_name
-- (alt_map_name is a map name replacement for ystra-lab)
function DailyNotes.GetNPCInfo(npc_id)

  -- cached
  local n_name,n_map,n_posx,n_posy,n_map_name,n_altname

  if DailyNotes.DB_NPC[npc_id] then
    n_name = DailyNotes.DB_NPC[npc_id].name
    n_map  = DailyNotes.DB_NPC[npc_id].map
    n_posx = DailyNotes.DB_NPC[npc_id].x
    n_posy = DailyNotes.DB_NPC[npc_id].y
    n_map_name = DailyNotes.DB_NPC[npc_id].map_name

    if n_name then
      if n_map_name then n_altname  = TEXT(n_map_name) end
      return n_name,n_map,n_posx,n_posy,n_altname
    end
  else
    n_name = TEXT("Sys"..npc_id.."_name")
    n_map  = 10000
    n_posx = 50
    n_posy = 50
  end


  if npc_id and type(npc_id)=="number" and ( NpcTrack_SearchNpcByDBID( npc_id ) >0 )then
    _, n_name, n_map, n_posx, n_posy = NpcTrack_GetNpc(1)
    n_posx = math.floor(n_posx*1000)/10
    n_posy = math.floor(n_posy*1000)/10

    DailyNotes.DB_NPC[npc_id] = {map=n_map,name=n_name,x=n_posx,y=n_posy,map_name=n_map_name}
  end

  if not n_name then
    n_name = "???"
    n_map  = 10000
    n_posx = 50
    n_posy = 50
  end

  if n_map_name then n_altname  = TEXT(n_map_name) end
  return n_name,n_map,n_posx,n_posy,n_altname
end


------------------------------
function DailyNotes.MakeItemCountString(qudata,icounts)

  local pre=nil
  local post=nil

  if (qudata.typ%10)~=2 then
    -- normal
    for iid,count in pairs(qudata.items or {}) do
      if pre then
        pre = pre..",".. (icounts[iid] or 0)
        post = post..","..count
      else
        pre = (icounts[iid] or 0)
        post = count
      end
    end
  else
    -- kill quest
    for iid,count in pairs(qudata.mobs or {}) do
      if pre then
        pre = pre..",".. (icounts[iid] or 0)
        post = post..","..count
      else
        pre = (icounts[iid] or 0)
        post = count
      end
    end
  end

  if not pre then return "??" end
  return pre.."/"..post,pre,post
end

------------------------------
function DailyNotes.EnableLootMessage(enable)
  if enable then
    for i=1,10 do
      local ch=getglobal("ChatFrame"..i)
      if ch and not ch.orig_addmessage then
        ch.orig_addmessage = ch.AddMessage
        ch.AddMessage = HookedAddMessage
      end
    end
    --DailyNotes.get_message = "^"..string.gsub(TEXT('SYS_GIVE_ITEM'),"<ITEM>","(|Hitem.*|r|h)")
    DailyNotes.get_message = string.gsub(TEXT('SYS_GIVE_ITEM'),"<ITEM>","(|Hitem.*|r|h)x?%s*%d*")
  else
    for i=1,10 do
      local ch=getglobal("ChatFrame"..i)
      if ch and ch.orig_addmessage then
        ch.AddMessage = ch.orig_addmessage
        ch.orig_addmessage = nil
      end
    end
  end
end


function HookedAddMessage(self,msg,r,g,b)

  if msg and DN_Options and DN_Options.itemcount then

    -- get item
    local ilink=string.match(msg,DailyNotes.get_message)
    if ilink then
      local itemid = Nyx.GetItemID(ilink)

      -- check if DQ item
      local _,quest = DailyNotes.FindQuestForItem(itemid)
      local target = quest and quest.items and quest.items[itemid]

      if target then
        local count = Nyx.GetTotalBagItemCount(itemid)
        msg = msg .." |cffffff00("..count .."/"..target..")"
      else
        if not DN_Options.dqionly then
          local count = Nyx.GetTotalBagItemCount(itemid)
          if count>1 then
            msg = msg .." |cffffff00("..count..")"
          end
        end
      end
    end
  end

  self.orig_addmessage(self, msg,r,g,b)
end


-----------------------------------------------------------------------
-- Tooltips
-----------------------------------------------------------------------
DailyNotes.orig_GameTooltipHyperLinkSetHyperLink = GameTooltipHyperLink["SetHyperLink"]
DailyNotes.orig_GameTooltipSetHyperLink = GameTooltip["SetHyperLink"]
DailyNotes.orig_GameTooltipSetBagItem = GameTooltip["SetBagItem"]
DailyNotes.orig_GameTooltipSetBankItem = GameTooltip["SetBankItem"]
DailyNotes.orig_GameTooltipSetAuctionBrowseItem = GameTooltip["SetAuctionBrowseItem"]
DailyNotes.orig_GameTooltipSetBootyItem = GameTooltip["SetBootyItem"]
DailyNotes.orig_GameTooltipSetActionItem=GameTooltip["SetActionItem"]
DailyNotes.orig_GameTooltipSetHouseItem= GameTooltip["SetHouseItem"]

-- hooks
function GameTooltip:SetBankItem( BankId )
    DailyNotes.orig_GameTooltipSetBankItem( self, BankId )
    DailyNotes.AppendTooltipText( self, GetBankItemLink( BankId ) )
end

function GameTooltip:SetBagItem( BankId )
    DailyNotes.orig_GameTooltipSetBagItem( self, BankId )
    DailyNotes.AppendTooltipText( self, GetBagItemLink( BankId ) )
end

function GameTooltip:SetHouseItem( ParentDBID, ButtonId )
    DailyNotes.orig_GameTooltipSetHouseItem( self, ParentDBID, ButtonId );

    if ( ParentDBID ) then
        DailyNotes.AppendTooltipText( self, Houses_GetItemLink( ParentDBID, ButtonId ) )
    end
end

function GameTooltip:SetHyperLink( itemLink )
    DailyNotes.orig_GameTooltipSetHyperLink( self, itemLink )
    DailyNotes.AppendTooltipText( self, itemLink )
end

function GameTooltipHyperLink:SetHyperLink( itemLink )
    DailyNotes.orig_GameTooltipHyperLinkSetHyperLink( self, itemLink )
    DailyNotes.AppendTooltipText( self, itemLink )
end

function GameTooltip:SetAuctionBrowseItem(id)
    DailyNotes.orig_GameTooltipSetAuctionBrowseItem(self, id)
    DailyNotes.AppendTooltipText( self, GetAuctionBrowseItemLink(id) )
end

function GameTooltip:SetBootyItem(id)
    DailyNotes.orig_GameTooltipSetBootyItem(self, id)
    DailyNotes.AppendTooltipText( self, GetBootyItemLink(id) )
end

-- main
function DailyNotes.AppendTooltipText(tooltip, itemLink )

    if not itemLink then return  end

    if not DN_Options.TT_items and not DailyNotes.InDialogTT then return end


    local itemid = Nyx.GetItemID(itemLink)
    local QID,quest
    if tooltip.QID then
        QID = tooltip.QID
        quest = DailyNotes.DB_Quests[QID]
    else
        QID,quest = DailyNotes.FindQuestForItem(itemid)
    end

    if QID then

        DailyNotes.AppendItemCount(tooltip, itemid, quest )
        DailyNotes.AppendQuestName(tooltip, QID, quest )
        DailyNotes.AppendQuestNPC(tooltip, quest)
        DailyNotes.AppendQuestText(tooltip, QID)
        DailyNotes.AppendReward(tooltip, quest)
        DailyNotes.AppendPrequest(tooltip, quest)
        DailyNotes.AppendInvolvedMobs(tooltip, quest)

        -- Debug flag
        if DN_Options.debug and quest.typ>20 then
          tooltip:AddLine( "|cffff0000(--UNTESTED-- Plz report)")
        end

    else
        if DN_Options.debug then
            tooltip:AddLine( "\n|cff909090ItemID: " .. Nyx.GetItemID(itemLink));
        end
    end
end

function DailyNotes.GetQuestTaker(qdata)
    local taker = qdata.taker or qdata.giver
    if type(taker)=="table" then
        return taker[1]
    end
    return taker
end

function DailyNotes.AppendItemCount(tooltip, itemid, quest )
    local max_completed, itemcounts = DailyNotes.GetQuestItems(quest)

    local taker = DailyNotes.GetQuestTaker(quest)
    tooltip:AddLine( string.format(DailyNotes.L.COUNT, itemcounts[itemid], quest.items[itemid], DailyNotes.GetNPCInfo(taker))) -- "8/10 for questname"

    if (max_completed>0) then

        tooltip:AddLine( string.format(DailyNotes.L.FIN, max_completed)) -- "Enough for: %i times"
    else
        local needs = quest.items[itemid]-itemcounts[itemid]
        if needs<0 then
            tooltip:AddLine( string.format(DailyNotes.L.NEEDSOTHER)) -- "Needs other item"
        else
            tooltip:AddLine( string.format(DailyNotes.L.NEED, needs)) -- "Needs: %i"
        end
    end
end

function DailyNotes.AppendQuestName(tooltip, QID, quest)
    local color = ""
    local mlvl, slvl = Nyx.GetPlayerLevel()
    if (quest.lvl > mlvl) then
        color = "|cffff0000"
        if (quest.lvl <= slvl) then
            color = "|cffff8080"
        end
    end
    tooltip:AddLine( color .. TEXT("Sys"..QID.."_name") .. " (Lvl " .. quest.lvl .. ")");
end

function DailyNotes.AppendQuestNPC(tooltip, quest)

    -- Where to get it
    local qtaker = DailyNotes.GetQuestTaker(quest)
    local last_start = nil

    local givers = quest.giver
    if type(givers)=="number" then givers={givers} end

    for _,value in ipairs(givers) do
      local n_name,n_map,n_posx,n_posy,n_altname = DailyNotes.GetNPCInfo(value)

      if n_name ~= last_start then

        if value == qtaker then
          tooltip:AddLine( string.format(DailyNotes.L.QSTARTEND, n_name)) -- Start: %s
          qtaker = nil
        else
          tooltip:AddLine( string.format(DailyNotes.L.QSTART, n_name)) -- Start: %s
        end

        if DN_Options.verbose or DailyNotes.InDialogTT then
          n_altname = n_altname or GetZoneLocalName(n_map)
          tooltip:AddLine( string.format(DailyNotes.L.QPOS, n_altname,n_posx,n_posy)) -- (%s: %g/%g)
        end
        last_start = n_name
      end
    end

    -- reward npc
    if qtaker and (DN_Options.verbose or DailyNotes.InDialogTT) then
      local n_name,n_map,n_posx,n_posy,n_sub = DailyNotes.GetNPCInfo(qtaker)

      tooltip:AddLine( string.format(DailyNotes.L.QEND, n_name)) -- Start: %s
      if DN_Options.verbose or DailyNotes.InDialogTT then
        n_altname = n_altname or GetZoneLocalName(n_map)
        tooltip:AddLine( string.format(DailyNotes.L.QPOS, n_altname,n_posx,n_posy)) -- (%s: %g/%g)
      end
    end
end

function DailyNotes.AppendQuestText(tooltip, QID)
    if DailyNotes.InDialogTT and DN_Options.verbose_quest then
        local text = Nyx.GetQuestBookText(QID)
        if text then
            tooltip:AddLine("\n")
            tooltip:AddLine(text)
            tooltip:AddLine("\n")
        end
    end
end

function DailyNotes.AppendReward(tooltip, quest)
    -- XP/Gold/Token (Note: TPs=floor(XP/10))
    tooltip:AddLine( string.format(DailyNotes.L.XP, quest.rew[1],quest.rew[2],quest.rew[3])) -- XP: %i   Gold/Token: %i/%i
end

function DailyNotes.AppendInvolvedMobs(tooltip, quest)

    if not DN_Options.debug or not quest.mobs then
        return
    end

    local mobs={}
    for mobid,val in pairs(quest.mobs) do
        if mobid and type(mobid)=="number" and ( NpcTrack_SearchNpcByDBID( mobid ) >0 )then
            local _, n_name, _, _, _ = NpcTrack_GetNpc(1)
            if n_name then
                table.insert(mobs,n_name)
            end
        end
    end

    if #mobs>0 then
        tooltip:AddLine("\nInvolved NPCs:")
        for _,n in pairs(mobs) do
            tooltip:AddLine("  "..n)
        end
    end
end

function DailyNotes.AppendPrequest(tooltip, quest)

    local preq = DailyNotes.GetRequiredPreQuest(quest)
    if not preq then return end

    tooltip:AddLine( DailyNotes.L.PREQUEST .. TEXT("Sys"..preq.."_name"));
end

function DailyNotes.GetRequiredPreQuest(quest)

    if type(quest.preq) == "table" then
        for _,q in ipairs(quest.preq) do
            if CheckQuest(q)~=2 then
                return q
            end
        end
    elseif quest.preq and CheckQuest(quest.preq)~=2 then
            return quest.preq
    end

end


function GameTooltip:SetActionItem(m_ButtonId)
    local result=DailyNotes.orig_GameTooltipSetActionItem (self,m_ButtonId)
    local icon_path = GetActionInfo(m_ButtonId)
    if icon_path then
      local itemname = _G[GameTooltip:GetName() .. "TextLeft1"]:GetText()
      if itemname then
        local iidx=Nyx.FindBagIndex(itemname,icon_path)
        if iidx then
            DailyNotes.AppendTooltipText( self, GetBagItemLink( iidx ) )
        end
      end
    end
    return result;
end
----------------------------------------
function DailyNotes.EnableUnitTooltip(enable)

    enable = enable or false
    if (DailyNotes.UnitToolTipIsEnabled or false) == (enable or false) then return end

    if enable then
        DailyNotes.UnitTooltipActivate()
    else
        DailyNotes.UnitTooltipDeactivate()
    end

    DailyNotes.UnitToolTipIsEnabled = enable
end

function DailyNotes.UnitTooltipActivate()

    if pbInfo and pbInfo.Tooltip and pbInfo.Tooltip.Scripts and pbInfo.Tooltip.Scripts.OnUpdate and  -- PBInfo Installed
      pbInfoSettings and pbInfoSettings["ENABLE"] -- PBInfo enabled
      and pbInfoSettings["MODIFYTOOLTIP"]         -- PBInfo tooltip enabled
      then
        DailyNotes.PBinfoHook = pbInfo.Tooltip.Scripts.OnUpdate
        pbInfo.Tooltip.Scripts.OnUpdate = DailyNotes.UPDATE_MOUSEOVER_UNIT
    else
      DailyNotesFrame:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
    end

    DailyNotes.DB_Mobs = {}
    for QID,quest in pairs(DailyNotes.DB_Quests) do
      for mobid,val in pairs(quest.mobs or {}) do
        if mobid and (quest.typ%10)~=2 then -- has mob info and is not kill-quest
          local n_name=TEXT("Sys"..mobid.."_name")
            DailyNotes.DB_Mobs[n_name] = DailyNotes.DB_Mobs[n_name] or {}

            if val==0 and quest.items then
                val = next(quest.items)
            end
            DailyNotes.DB_Mobs[n_name][QID]=val
        end
      end
    end
end

function DailyNotes.UnitTooltipDeactivate()
    if DailyNotes.UnitToolTipIsEnabled then
        if DailyNotes.PBinfoHook then
            pbInfo.Tooltip.Scripts.OnUpdate = DailyNotes.PBinfoHook
        else
            DailyNotesFrame:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
        end
    end

    DailyNotes.DB_Mobs = nil
end

function DailyNotes.UPDATE_MOUSEOVER_UNIT()

    if DailyNotes.PBinfoHook then
        DailyNotes.PBinfoHook()
    end

    if (not UnitCanAttack("player", "mouseover") or UnitIsPlayer("mouseover")) then
        return
    end

  local Qlist = DailyNotes.DB_Mobs and DailyNotes.DB_Mobs[UnitName("mouseover")]
  if Qlist then

    GameTooltip:AddLine("\n")

    for QID, item_id in pairs(Qlist) do

        if item_id>1000 then
            local item_count = Nyx.GetTotalBagItemCount(item_id)
            local item_target_count = DailyNotes.DB_Quests[QID].items and DailyNotes.DB_Quests[QID].items[item_id]

            GameTooltip:AddLine( string.format(DailyNotes.L.DROPSITEM,TEXT("Sys"..item_id.."_name")) ..
                             " ("..tostring(item_count) .."/"..tostring(item_target_count)..")")

        else
            GameTooltip:AddLine( string.format(DailyNotes.L.INVOLVED,TEXT("Sys"..QID.."_name")) )
        end
    end
  end

end



-----------------------------------------------------------------------
--  Bulletin Board / Quest givers
-----------------------------------------------------------------------
DailyNotes.orig_SpeakFrame_LoadQuest = SpeakFrame_LoadQuest
DailyNotes.orig_SpeakFrame_LoadQuestDetail = SpeakFrame_LoadQuestDetail

function SpeakFrame_LoadQuest()
    DailyNotes.orig_SpeakFrame_LoadQuest()
    DailyNotes.ChangeSpeakFrame()
end

function SpeakFrame_LoadQuestDetail( Quitmode )
    DailyNotes.orig_SpeakFrame_LoadQuestDetail( Quitmode )
    DailyNotes.ChangeSpeakFrame(1)
end


function DailyNotes.ChangeSpeakFrame(is_in_detail_view)

    local auto_do_quest = nil

    local counter_new_quests = 0
    local counter_finished_quests = 0

    for i= 1,g_SpeakFrameData.count do

        -- NEW Quests
        if g_SpeakFrameData.option[i].type==DF_IconType_NewQuest then

            counter_new_quests = counter_new_quests +1

            local QID, qdata = DailyNotes.SF_TestNewQuest(i, is_in_detail_view)
            if QID then

                local count, itemcounts = DailyNotes.GetQuestItems(qdata)
                DailyNotes.SF_ColorNewQuest(i, is_in_detail_view, QID, qdata, count, itemcounts)

                if DailyNotes.SF_IsAutoAcceptNewQuest(QID, count, qdata) then
                    if not auto_do_quest or auto_do_quest.xp < qdata.rew[1] then
                        auto_do_quest = { index = counter_new_quests, type = DF_IconType_NewQuest, xp = qdata.rew[1] }
                    end
                end
            end

        -- Unfinished Quests
        elseif g_SpeakFrameData.option[i].type==DF_IconType_UnfinishQuest then

            local QID, qdata = DailyNotes.SF_TestNewQuest(i, is_in_detail_view)
            if QID then
                local count, itemcounts = DailyNotes.GetQuestItems(qdata)
                DailyNotes.SF_ColorNewQuest(i, is_in_detail_view, QID, qdata, count, itemcounts)
            end

        -- Finished Quest
        elseif g_SpeakFrameData.option[i].type==DF_IconType_FinishedQuest then

            counter_finished_quests = counter_finished_quests +1

            local QID, qdata = DailyNotes.SF_TestFinishedQuest(i, is_in_detail_view)
            if QID then

                DailyNotes.SF_ColorFinishedQuest(i, is_in_detail_view, QID, qdata)

                if not auto_do_quest and DailyNotes.SF_IsAnAutoQuest(QID) then
                    auto_do_quest = { index = counter_finished_quests, type = DF_IconType_FinishedQuest }
                end
            end
        end
    end

    -- 'repaint'
    SpeakFrame_Show()

    if not IsAltKeyDown() and not IsCtrlKeyDown() then
        DailyNotes.SF_AutoProcessQuest(is_in_detail_view, auto_do_quest)
    end
end

function DailyNotes.SF_TestNewQuest(index, detail)

    local name = g_SpeakFrameData.option[index].title
    if detail and name:find("^"..SPEAKFRAME_QUEST_ACCEPT)  then
        name = name:sub(SPEAKFRAME_QUEST_ACCEPT:len()+1)
    end

    return DailyNotes.FindQuestByName(name, GetSpeakObjName())
end

function DailyNotes.SF_TestFinishedQuest(index, detail)

    local name = g_SpeakFrameData.option[index].title
    if detail and name:find("^"..SPEAKFRAME_QUEST_COMPLETE) then
        name = name:sub(SPEAKFRAME_QUEST_COMPLETE:len()+1)
    end

    return DailyNotes.FindQuestByName(name, GetSpeakObjName())
end


function DailyNotes.SF_ColorNewQuest(i, detail, QID, qdata, count, itemcounts)

    local kind = qdata.typ%10

    local color = ""
    if count>0 then
        color="|cff40ff40"
    elseif kind==2 then -- kill quest
        color = "|cffb0b0ff"
    end

    g_SpeakFrameData.option[i].title = color.."["..qdata.lvl.."] "..TEXT("Sys"..QID.."_name").." ("..DailyNotes.MakeItemCountString(qdata,itemcounts)..")"


    if qdata.typ>20 and DN_Options.debug then
        g_SpeakFrameData.option[i].title= "|cffffffff"..g_SpeakFrameData.option[i].title
    end

end

function DailyNotes.SF_ColorFinishedQuest(i, detail, QID, qdata)
    local count, itemcounts = DailyNotes.GetQuestItems(qdata)

    g_SpeakFrameData.option[i].title= "|cff40ff40"..TEXT("Sys"..QID.."_name")

    kind = qdata.typ%10
    if kind ~= 2 and count>0 then
        g_SpeakFrameData.option[i].title= g_SpeakFrameData.option[i].title.." ("..DailyNotes.MakeItemCountString(qdata,itemcounts)..")"
    end
end

function DailyNotes.SF_IsAnAutoQuest(QID)
    local options = DN_Char_Options

    return options.autoquest       -- auto quest enabled
       and options.aq_accept
       and options.aq_accept[QID]  -- quest marked for autoquest
end

function DailyNotes.SF_IsAutoAcceptNewQuest(QID, count, qdata)

    if DailyNotes.SF_IsAnAutoQuest(QID) then

        -- kill, gather or special quest
        local kind = qdata.typ%10
        if kind==2 or kind==4 or kind==5 then return true end

        -- enough material
        if count>0 or not DN_Options.takefilter then return true end
    end
end



function DailyNotes.SF_AutoProcessQuest(in_detail_view, do_quest)

    if do_quest then
        -- NOTE: removed 'cause quests won't appear at the board
        -- catch: 10 dailies already done
        --~  local ddone , dmax = Daily_count();
        --~  if do_quest_type==DF_IconType_NewQuest and ddone>=dmax then
        --~    do_quest = nil
        --~    DEFAULT_CHAT_FRAME:AddMessage(DailyNotes.L.ERR_DAILIES)
        --~  end

        -- catch: 30 quest open
        if do_quest.type==DF_IconType_NewQuest and (g_NumTotalQuest or 0) >=30 then
            do_quest = nil
            DEFAULT_CHAT_FRAME:AddMessage(DailyNotes.L.ERR_QUESTS)
        end
    end

    if do_quest then
        -- in detail view: accept/complete
        if in_detail_view==1 then
            if do_quest.type==DF_IconType_NewQuest  then
                AcceptQuest()
            else
                CompleteQuest()
            end
            SpeakFrame_Hide()
        else
        -- in list view: request detail view
            OnClick_QuestListButton(do_quest.type, do_quest.index)
        end
    end

end


-----------------------------------------------------------------------
-- DN Frame
-----------------------------------------------------------------------
DailyNotes.fct_compare={
 -- count
 function (a,b)  return a[4]>b[4] end,
 -- target
 function (a,b)  return false end,
 -- name
 function (a,b)  return TEXT("Sys"..a[1].."_name")<TEXT("Sys"..b[1].."_name") end,
 -- level
 function (a,b)  if DN_Options.showcharlevel then return a[2].rlvl>b[2].rlvl end return a[2].lvl>b[2].lvl end,
 -- zone
 function (a,b)  return GetZoneLocalName(a[2].zone)<GetZoneLocalName(b[2].zone) end,
 -- item
 function (a,b)  return DailyNotes.GetQuestItemName(a[2])<DailyNotes.GetQuestItemName(b[2]) end,
 -- xp
 function (a,b)  return a[2].rew[1]>b[2].rew[1] end,
}

DailyNotes.fct_filter_status={
  function(fin,count) return true end,
  function(fin,count) return fin>0 end,
  function(fin,count) return count>0 end
}

function DailyNotes.GetQuestItemName(qdata)
  local itemid = next(qdata.items or {})
  if itemid then return TEXT("Sys"..itemid.."_name") end
  local mobid = next(qdata.mobs or {})
  if mobid then return DailyNotes.GetNPCInfo(mobid) end
  return ""
end


---------------------------------
function DailyNotes.UpdateList()

  -- Get Options
  local status_filter = DailyNotes.fct_filter_status[UIDropDownMenu_GetSelectedID(DNFFilterCountDropDown)] or DailyNotes.fct_filter_status[1]

  local nzone_filter = DN_Char_Options.FilterZone or -3
  if nzone_filter == -3 then
     nzone_filter = GetCurrentWorldMapID()
  elseif nzone_filter == -2 then
     nzone_filter = nil
  end

  DN_Char_Options.FilterStatus = UIDropDownMenu_GetSelectedID(DNFFilterCountDropDown)


  -- Get filtered list
  local mlvl, slvl = Nyx.GetPlayerLevel()
  local total_qcount=0

  DailyNotes.view_items={}
  for QID,qdata in pairs(DailyNotes.DB_Quests) do
    local finished,itemcounts,count = DailyNotes.GetQuestItems(qdata)
    total_qcount = total_qcount + finished
    if not nzone_filter or (qdata.zone==nzone_filter or DailyNotes.DB_Zones[nzone_filter]==qdata.zone or DailyNotes.DB_Zones[qdata.zone]==nzone_filter) then -- zone filter
      -- status filter (finished, started, none)
      if status_filter(finished,count) then
         -- main class level filter (can take quest)
        if (DN_Options.filtermain and mlvl>=qdata.rlvl) or
           -- second class level filter (can take quest or main can take and second can reward it)
           (DN_Options.filtersec  and (slvl>=qdata.lvl or (slvl>=qdata.lvl-10 and mlvl>=qdata.rlvl))) or
           -- (or no level filter)
           (not DN_Options.filtermain and not DN_Options.filtersec)  then
          table.insert(DailyNotes.view_items,{QID,qdata,finished,count,itemcounts})
        end
      end
    end
  end

  -- Sort
  if DN_Options.Sort and DN_Options.Sort<0 and DailyNotes.fct_compare[-DN_Options.Sort] then
    table.sort(DailyNotes.view_items, function (a,b) return DailyNotes.fct_compare[-DN_Options.Sort](b,a) end)
  else
    table.sort(DailyNotes.view_items, DailyNotes.fct_compare[DN_Options.Sort] or DailyNotes.fct_compare[1])
  end


  -- Update view
  if #DailyNotes.view_items > DailyNotes.MAX_LIST_ENTRIES then
    DailyNotesListScrollBar:Show();
    DailyNotesListScrollBar:SetMinMaxValues(0, #DailyNotes.view_items-DailyNotes.MAX_LIST_ENTRIES);
  else
    DailyNotesListScrollBar:Hide();
    DailyNotesListScrollBar:SetMinMaxValues(0, 0);
  end

  -- Quests total; finished quest
  DailyNotesListSize:SetText(#DailyNotes.view_items.." / ".. DailyNotes.Quest_count)
  DailyNotesFinAble:SetText(total_qcount)

  DailyNotes.UpdateListView()
end



function DailyNotes.List_Header_OnClick(this)

  local id = this:GetID()
  DailyNotesListCountHeaderSortIcon:SetFile("");
  DailyNotesListTargetHeaderSortIcon:SetFile("");
  DailyNotesListLevelHeaderSortIcon:SetFile("");
  DailyNotesListNameHeaderSortIcon:SetFile("");
  DailyNotesListZoneHeaderSortIcon:SetFile("");
  DailyNotesListItemHeaderSortIcon:SetFile("");
  DailyNotesListXPHeaderSortIcon:SetFile("");

  if(DN_Options.Sort == id or DN_Options.Sort == -id ) then
    DN_Options.Sort = -DN_Options.Sort
  else
    DN_Options.Sort = id;
  end

  if DN_Options.Sort>0 then
    getglobal(this:GetName() .. "SortIcon"):SetFile("Interface/chatframe/chatframe-scrollbar-buttondown")
  else
    getglobal(this:GetName() .. "SortIcon"):SetFile("Interface/chatframe/chatframe-scrollbar-buttonup")
  end

  DailyNotes.UpdateList()
end

----------------------------
function DailyNotes.OnShow()
    local status_filter = DN_Char_Options.FilterStatus or 1
    UIDropDownMenu_SetSelectedID(DNFFilterCountDropDown, status_filter)

    local count_text = status_filter
    if     count_text==2 then count_text = DailyNotes.L.COUNT_FIN
    elseif count_text==3 then count_text = DailyNotes.L.COUNT_START
    else   count_text = DailyNotes.L.COUNT_ALL
    end
    DNFFilterCountDropDownText:SetText(tostring(count_text))


    -- set zone
    local zone_text=""
    local zonenum = DN_Char_Options.FilterZone or -3
    if zonenum==-2 then zone_text=DailyNotes.L.ZONE_ALL
    elseif zonenum==-3 then zone_text=DailyNotes.L.ZONE_CUR
    else
        if zonenum == 670 then
            zone_text = DailyNotes.L.ZONE_YLAB
        else
        zone_text=GetZoneLocalName(zonenum)
        end
    end
    UIDropDownMenu_SetSelectedName(DNFFilterZoneDropDown, zone_text)
    DNFFilterZoneDropDownText:SetText(zone_text)

    DailyNotesFilterLevel1:SetChecked(DN_Options.filtermain)
    DailyNotesFilterLevel2:SetChecked(DN_Options.filtersec)
    DailyNotesAutoQuest:SetChecked(DN_Char_Options.autoquest)

    DailyNotes.UpdateAQButtons()
    DailyNotes.UpdateList()
end

function DailyNotes.OnHide()
    DailyNotes.view_items=nil
end

----------------------------
function DailyNotes.UpdateListView()

    local index = DailyNotesListScrollBar:GetValue()
    local entry,key,button

    local mlvl, slvl = Nyx.GetPlayerLevel()

    local button,QID,qdata,qname

    for i = 1, DailyNotes.MAX_LIST_ENTRIES do
        button = getglobal("DailyNotesListItem"..i);


        if DailyNotes.view_items[index+i] then
            QID = DailyNotes.view_items[index+i][1]
            qdata = DailyNotes.view_items[index+i][2]
            qname = TEXT("Sys"..QID.."_name")

            buttonName = button:GetName();

            local _,count, target = DailyNotes.MakeItemCountString(qdata,DailyNotes.view_items[index+i][5])
            _G[buttonName.."Counter"]:SetText(count)
            _G[buttonName.."Target"]:SetText(target)
            if not count and not target then
                _G[buttonName.."Splitter"]:Hide()
            else
                _G[buttonName.."Splitter"]:Show()
            end

            if DN_Options.showcharlevel then
                getglobal(buttonName.."Level"):SetText(qdata.rlvl)
            else
                getglobal(buttonName.."Level"):SetText(qdata.lvl)
            end

            getglobal(buttonName.."Name"):SetText(qname)
            getglobal(buttonName.."Zone"):SetText(GetZoneLocalName(qdata.zone))
            getglobal(buttonName.."Item"):SetText(DailyNotes.GetQuestItemName(qdata))
            getglobal(buttonName.."XP"):SetText(qdata.rew[1])

            -- icon
            if (math.floor(qdata.typ/10)%2)==1 then
                getglobal(buttonName.."IconOnce"):Show()
            else
                getglobal(buttonName.."IconOnce"):Hide()
            end

            local typ = qdata.typ%10
            if     typ==2 then getglobal(buttonName.."Icon"):SetTexCoord(0.25, 0.375,0,1) -- kill
            elseif typ==4 then getglobal(buttonName.."Icon"):SetTexCoord(0.375,0.5,  0,1) -- gather
            elseif typ==6 then getglobal(buttonName.."Icon"):SetTexCoord(0.626,0.76, 0,1) -- epic
            else               getglobal(buttonName.."Icon"):SetTexCoord(0.125,0.25, 0,1) -- normal
            end

            -- QuestStatus
            local cq = CheckQuest(QID)
            local icondone = getglobal(buttonName.."IconDone")
            if cq==1 then
                icondone:Show() -- open
                icondone:SetTexCoord(0.5, 0.5625,0,1)
            elseif cq==2 then
                icondone:Hide() -- done
            else
                icondone:Show() -- not done
                icondone:SetTexCoord(0.5625, 0.625,0,1)
            end

            -- color
            r,g,b=1,1,1
            if DailyNotes.view_items[index+i][3]>0 then r,g,b=0,1,0 -- FINISHED
            elseif DailyNotes.GetRequiredPreQuest(qdata) then r,g,b=1,0.3,0.3 -- PRE.QUEST
            elseif DN_Options.debug and qdata.typ>20 then r,g,b=1,0,1 -- NEED TEST
            elseif mlvl<qdata.rlvl then r,g,b=0.6,0.6,0.6
            end

            getglobal(buttonName.."Counter"):SetColor(r,g,b)
            getglobal(buttonName.."Target"):SetColor(r,g,b)
            getglobal(buttonName.."Level"):SetColor(r,g,b)
            getglobal(buttonName.."Name"):SetColor(r,g,b)
            getglobal(buttonName.."Zone"):SetColor(r,g,b)
            getglobal(buttonName.."Item"):SetColor(r,g,b)
            getglobal(buttonName.."XP"):SetColor(r,g,b)

            if DN_Char_Options.autoquest then
                getglobal(buttonName.."Check"):Show()
                getglobal(buttonName.."Check"):SetChecked(DailyNotes.SF_IsAnAutoQuest(QID))
            else
                getglobal(buttonName.."Check"):Hide()
            end

            button:Show()
        else
            button:Hide()
        end
    end

    DailyNotes.List_UpdateDoAble()

    if DailyNotes.cur_highlit then
        local c = DailyNotes.cur_highlit
        DailyNotes.List_Item_OnLeave(c)
        DailyNotes.List_Item_OnEnter(c)
    end

end


------------
-- List tooltip
function DailyNotes.List_Item_OnEnter(this)

  if not DN_Options.TT_dialog then return end

  DailyNotes.cur_highlit = this

  local highlightText = getglobal(this:GetName().."Highlight")
  highlightText:SetColor(1, 1, 0)
  highlightText:Show()

  local index = DailyNotesListScrollBar:GetValue() + this:GetID()
  if not DailyNotes.view_items[index] then return end

  local QID = DailyNotes.view_items[index][1]
  local qdata = DailyNotes.view_items[index][2]

  DailyNotes.InDialogTT = true

  -- Show the tooltip
  GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT", 0, -16)
  local ty=qdata.typ%10

    --------------------------------
    -- Quest with Item
    if qdata.items then
        local itemid,_ = next(qdata.items)
        GameTooltip.QID = QID
        GameTooltip:SetHyperLink( Nyx.CreateItemLink( itemid ) )
        GameTooltip:Show()
        GameTooltip.QID = nil

    --------------------------------
    -- Quest with Mobs
    elseif qdata.mobs then

        GameTooltip:SetText("")
        local first=true
        for mobid,mobcount in pairs(qdata.mobs or {}) do
          if first then
            GameTooltip:SetText(DailyNotes.GetNPCInfo(mobid))
            GameTooltip:AddLine("")
            first = nil
          else
            GameTooltip:AddLine("+ " .. DailyNotes.GetNPCInfo(mobid))
          end
          local qtaker = DailyNotes.GetQuestTaker(qdata)
          GameTooltip:AddLine(string.format(DailyNotes.L.KILLCOUNT, mobcount, DailyNotes.GetNPCInfo(qtaker))) -- "10 for taker"
        end

        DailyNotes.AppendQuestName(GameTooltip, QID, qdata)
        DailyNotes.AppendQuestNPC(GameTooltip, qdata)
        DailyNotes.AppendQuestText(GameTooltip, QID)
        DailyNotes.AppendReward(GameTooltip, qdata)
        DailyNotes.AppendPrequest(GameTooltip, qdata)
        DailyNotes.AppendInvolvedMobs(GameTooltip, qdata)

        -- Debug flag
        if DN_Options.debug and qdata.typ>20 then
            GameTooltip:AddLine( "|cffff0000(--UNTESTED-- Plz report)")
        end

        GameTooltip:Show()

    --------------------------------
    -- No mob or item
    else

        GameTooltip:SetText("")
        DailyNotes.AppendQuestName(GameTooltip, QID, qdata)
        DailyNotes.AppendQuestNPC(GameTooltip, qdata)
        DailyNotes.AppendQuestText(GameTooltip, QID)
        DailyNotes.AppendReward(GameTooltip, qdata)
        DailyNotes.AppendPrequest(GameTooltip, qdata)

        GameTooltip:Show()

        -- Debug flag
        if DN_Options.debug and qdata.typ>20 then
            GameTooltip:AddLine( "|cffff0000(--UNTESTED-- Plz report)")
        end
    end

    DailyNotes.InDialogTT = nil
end


function DailyNotes.List_Item_OnLeave(this)
    DailyNotes.cur_highlit = nil
    local highlightText = getglobal(this:GetName().."Highlight")
    highlightText:SetColor(1, 1, 1)
    highlightText:Hide()
    GameTooltip:Hide()
end

------------
-- List Context menu
function DailyNotes.List_Item_OnClick(this, key)
  if( key == "RBUTTON" or  key == "LBUTTON")then
    GameTooltip:Hide()
    DNFrameItemMenu.ItemIndex = DailyNotesListScrollBar:GetValue() + this:GetID()
    local QID = DailyNotes.view_items[DNFrameItemMenu.ItemIndex][1]
    ToggleDropDownMenu(DNFrameItemMenu, 1,TEXT("Sys"..QID.."_name"),"cursor", 1 ,1 );
  end
end

function DailyNotes.ContextFindNPC( id )
  ShowUIPanel( NpcTrackFrame )
  NpcTrackFrame_InitializeNpcList(id)
  NpcTrackFrame:ResetFrameOrder()
  --NpcTrackFrame_QuickTrackByNpcID(id)
end

function DailyNotes.ShowContextMenu( this )

  local info = {};

  if( UIDROPDOWNMENU_MENU_LEVEL == 1 and this.ItemIndex ) then

    local QID = DailyNotes.view_items[this.ItemIndex][1]
    local qdata = DailyNotes.view_items[this.ItemIndex][2]


    -- Start NPC
    local qtaker = DailyNotes.GetQuestTaker(qdata)
    local last_start = nil

    local givers = qdata.giver
    if type(givers)=="number" then givers={givers} end

    for _,value in ipairs(givers) do
      local n_name,n_map,n_posx,n_posy = DailyNotes.GetNPCInfo(value)

      if n_name ~= last_start then

        if value == qtaker then
          qtaker = nil

          info = {}
          info.text   = DailyNotes.L.FIND_NPC..n_name
          info.notCheckable = 1
          info.func   = function() DailyNotes.ContextFindNPC( value ) end
          info.value  = 1
          UIDropDownMenu_AddButton(info)
        else
          info = {}
          info.text   = DailyNotes.L.FIND_START..n_name
          info.notCheckable = 1
          info.func   = function() DailyNotes.ContextFindNPC( value ) end
          info.value  = 1
          UIDropDownMenu_AddButton(info)
        end

        last_start = n_name
      end
    end

    for mobid,val in pairs(qdata.mobs or {}) do
      if mobid and type(mobid)=="number" and ( NpcTrack_SearchNpcByDBID( mobid ) >0 )then
        local _, n_name, _, _, _ = NpcTrack_GetNpc(1)
        info = {}
        info.text   = DailyNotes.L.FIND_MOB..n_name
        info.notCheckable = 1
        info.func   = function() DailyNotes.ContextFindNPC( mobid ) end
        info.value  = 1
        UIDropDownMenu_AddButton(info)
      end
    end

    -- reward npc
    if qtaker then
      local n_name,n_map,n_posx,n_posy = DailyNotes.GetNPCInfo(qtaker)

      info = {}
      info.text   = DailyNotes.L.FIND_END..n_name
      info.notCheckable = 1
      info.func   = function() DailyNotes.ContextFindNPC( qtaker ) end
      info.value  = 1
      UIDropDownMenu_AddButton(info)
    end

    info = {}
    info.text   = TEXT("C_CANCEL")
    info.notCheckable = 1
    info.func   = function() end
    info.value  = 6
    UIDropDownMenu_AddButton(info)

  end
end
------------
-- Count filter
function DailyNotes.FCountDropDown_OnLoad(this)
  UIDropDownMenu_SetWidth(this, 80)
  UIDropDownMenu_Initialize(this, DailyNotes.FCountDropDown_Show)
  UIDropDownMenu_SetSelectedID(DNFFilterCountDropDown, 0)
end

function DailyNotes.FCountDropDown_Show()
  local info;
  info = {};
  info.text = DailyNotes.L.COUNT_ALL
  info.func = DailyNotes.FilterCountDropDown_Click;
  UIDropDownMenu_AddButton(info);

  info = {};
  info.text = DailyNotes.L.COUNT_FIN
  info.func = DailyNotes.FilterCountDropDown_Click;
  UIDropDownMenu_AddButton(info);

  info = {};
  info.text = DailyNotes.L.COUNT_START
  info.func = DailyNotes.FilterCountDropDown_Click;
  UIDropDownMenu_AddButton(info);
end


function DailyNotes.FilterCountDropDown_Click(button)
  UIDropDownMenu_SetSelectedID(DNFFilterCountDropDown, button:GetID())
  DailyNotes.UpdateList()
end


------------
-- zone filter
function DailyNotes.FZoneDropDown_OnLoad(this)
  UIDropDownMenu_SetWidth(this, 100)
  UIDropDownMenu_Initialize(this, DailyNotes.FZoneDropDown_Show)
end


local function IsInZone(zone, parent)
    if zone==parent then return true end

    local p = DailyNotes.DB_Zones[zone]
    if not p then return false end
    if p==parent then return true end
    return IsInZone(p, parent)
end

function DailyNotes.FZoneDropDown_Show()
    local info
    local zonenum = DN_Char_Options.FilterZone or -3

    if( UIDROPDOWNMENU_MENU_LEVEL == 1 ) then
        info = {}
        info.func = DailyNotes.FilterZoneDropDown_Click
        info.text = DailyNotes.L.ZONE_ALL
        info.value = -2
        info.checked = zonenum==info.value
        UIDropDownMenu_AddButton(info)

        info = {}
        info.func = DailyNotes.FilterZoneDropDown_Click
        info.text = DailyNotes.L.ZONE_CUR
        info.value = -3
        info.checked = zonenum==info.value
        UIDropDownMenu_AddButton(info)

        info = {}
        info.text = TEXT("ZONE_YGGNO LAND")
        info.hasArrow = 1
        info.value = 9998
        info.checked = zonenum==info.value
        UIDropDownMenu_AddButton(info)

        info = {}
        info.text = TEXT("ZONE_SAVILLEPLAINS")
        info.hasArrow = 1
        info.value = 9997
        info.checked = zonenum==info.value
        UIDropDownMenu_AddButton(info)

        info = {}
        info.text = TEXT("UI_WORLDMAP_OTHER")
        info.hasArrow = 1
        info.value = 9999
        info.checked = zonenum==info.value
        UIDropDownMenu_AddButton(info)

    elseif( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then

        local p_zone = UIDROPDOWNMENU_MENU_VALUE

        local zones={}
        local zones_in={}
        for _,d in pairs(DailyNotes.DB_Quests) do
            if not zones_in[d.zone] and IsInZone(d.zone, p_zone) then
                zones_in[d.zone]=1
                table.insert(zones,{d.zone,GetZoneLocalName(d.zone) or ""})
            end
        end

        -- ystra-lab
        if p_zone==9998 then
            table.insert(zones,{670,DailyNotes.L.ZONE_YLAB})
        end


        table.sort(zones,function (a,b) return a[2]<b[2] end)

        for _,n in ipairs(zones) do
            info = {}
            info.func = DailyNotes.FilterZoneDropDown_Click
            info.text = n[2]
            info.value= n[1]
            info.checked = zonenum==info.value
            UIDropDownMenu_AddButton(info,2)
        end
    end
end


function DailyNotes.FilterZoneDropDown_Click(button)
    UIDropDownMenu_SetSelectedValue(DNFFilterZoneDropDown, button.value)
    DNFFilterZoneDropDownText:SetText(button:GetText())
    DN_Char_Options.FilterZone = button.value
    DailyNotes.UpdateList()
    CloseDropDownMenus()
end

------------
-- levels
function DailyNotes.FilterLevel(btn)
  DN_Options.filtermain = DailyNotesFilterLevel1:IsChecked()
  DN_Options.filtersec  = DailyNotesFilterLevel2:IsChecked()

  DailyNotes.UpdateList()
end


------------
-- AutoQuest
function DailyNotes.AutoQuest(btn)
  DN_Char_Options.autoquest = DailyNotesAutoQuest:IsChecked()
  DailyNotes.UpdateAQButtons()
  DailyNotes.UpdateListView()
end


function DailyNotes.UpdateAQButtons()
  if DN_Char_Options.autoquest then
    DailyNoteAQClear:Show()
    DailyNoteAQAll:Show()
    DailyNoteAQKill:Show()
    DailyNoteAQGather:Show()
    DailyNoteAQKill:UnlockPushed()
    DailyNoteAQGather:UnlockPushed()

    -- check for 'all kill' and 'all gather'
    DailyNotes.AQ_AllKillMarked=1
    DailyNotes.AQ_AllGatherMarked=1
    if DN_Char_Options.aq_accept then
      for QID,qdata in pairs(DailyNotes.DB_Quests) do
        if (qdata.typ%10)==2 then
          DailyNotes.AQ_AllKillMarked = DN_Char_Options.aq_accept[QID] and DailyNotes.AQ_AllKillMarked
        elseif (qdata.typ%10)==4 then
          DailyNotes.AQ_AllGatherMarked = DN_Char_Options.aq_accept[QID] and DailyNotes.AQ_AllGatherMarked
        end
      end
    else
      DailyNotes.AQ_AllKillMarked=nil
      DailyNotes.AQ_AllGatherMarked=nil
    end

    if DailyNotes.AQ_AllKillMarked   then DailyNoteAQKill:LockPushed() end
    if DailyNotes.AQ_AllGatherMarked then DailyNoteAQGather:LockPushed() end

  else
    DailyNoteAQClear:Hide()
    DailyNoteAQAll:Hide()
    DailyNoteAQKill:Hide()
    DailyNoteAQGather:Hide()
  end
end


function DailyNotes.AutoQuestSetCheck(btn,ischecked)
    local index = DailyNotesListScrollBar:GetValue() + btn:GetID()

    local QID = DailyNotes.view_items[index][1]
    if QID then
        DN_Char_Options.aq_accept = DN_Char_Options.aq_accept or {}
        if ischecked then
            DN_Char_Options.aq_accept[QID] = 1
        else
            DN_Char_Options.aq_accept[QID] = nil
        end

        DailyNotes.UpdateAQButtons()
    end
end

function DailyNotes.AutoQuestClear()
    DN_Char_Options.aq_accept={}
    DailyNotes.MarkAutoQuests(function() return false end)
end

function DailyNotes.AutoQuestSelectAll()
    DN_Char_Options.aq_accept={}
    DailyNotes.MarkAutoQuests(function() return true end, true)
end

function DailyNotes.AutoQuestSelectKill()
    local m = not DailyNotes.AQ_AllKillMarked
    if not m then m=nil end

    DailyNotes.MarkAutoQuests(function(qdata) return (qdata.typ%10)==2 end, m)
end

function DailyNotes.AutoQuestSelectGather()
    local m = not DailyNotes.AQ_AllGatherMarked
    if not m then m=nil end

    DailyNotes.MarkAutoQuests(function(qdata) return (qdata.typ%10)==4 end, m)
end

function DailyNotes.MarkAutoQuests(filter_fct, mark)
    local accept_list = DN_Char_Options.aq_accept or {}

    for QID,qdata in pairs(DailyNotes.DB_Quests) do
        if filter_fct(qdata) then
            accept_list[QID]= mark
        end
    end

    DN_Char_Options.aq_accept = accept_list

    DailyNotes.UpdateAQButtons()
    DailyNotes.UpdateListView()
end

------------
-- Daily counter
function DailyNotes.QUEST_MSG_DAILYRESET()
  DailyNotes.List_UpdateDoAble()
end

function DailyNotes.QUEST_MSG_DAILY_OVERDONE()
  DailyNotes.List_UpdateDoAble()
end

function DailyNotes.RESET_QUESTTRACK()
  DailyNotes.List_UpdateDoAble()
end


function DailyNotes.List_UpdateDoAble()
  local done , dmax2 = Daily_count()

  if done<DailyNotes.Last_daily_count and done<dmax2 then
    SendSystemMsg(DailyNotes.L.DAILIES_RESET)
    DEFAULT_CHAT_FRAME:AddMessage(DailyNotes.L.DAILIES_RESET, 1,1,0)
  end

  DailyNotes.Last_daily_count = done
  DailyNotesDoAble:SetText(done.."/"..dmax2)
end


-----------------------------------------------------------------------
-- Configuration Dialog
-----------------------------------------------------------------------

function DailyNotes.Show_Options()
    DailyNotes.Config_Load()
    ToggleUIFrame(DailyNotesConfigFrame)

end

function DailyNotes.Config_Load()
    DailyNotesCharLevel:SetChecked(DN_Options.showcharlevel)
    DailyNotesTaker:SetChecked(DN_Options.takefilter)
    DailyNotesVerbose:SetChecked(DN_Options.verbose)
    DailyNotesVerboseQuest:SetChecked(DN_Options.verbose_quest)
    DailyNotesCounterDisplay:SetChecked(DN_Options.itemcount)
    DailyNotesCounterDQ:SetChecked(DN_Options.dqionly)
    DailyNotesDebug:SetChecked(DN_Options.debug)
    DailyNotesTT_Mobs:SetChecked(DN_Options.TT_mobs)
    DailyNotesTT_Items:SetChecked(DN_Options.TT_items)
    DailyNotesTT_Dialog:SetChecked(DN_Options.TT_dialog)
    DailyNotesZbagHook:SetChecked(DN_Options.ZBag_Hook)

    if DN_Options.itemcount then
        DailyNotesCounterDQ:Enable()
    else
        DailyNotesCounterDQ:Disable()
    end


    if Nyx.VersionToNumber( ZBAG_VERSION ) > 13000 then
        DailyNotesZbagHook:Show()
    else
        DailyNotesZbagHook:Hide()
    end

    --[===[@debug@
    DailyNotesDebug:Show()
    --@end-debug@]===]
end

function DailyNotes.Config_Save()

  DN_Options.showcharlevel = DailyNotesCharLevel:IsChecked()
  DN_Options.takefilter = DailyNotesTaker:IsChecked()
  DN_Options.verbose = DailyNotesVerbose:IsChecked()
  DN_Options.verbose_quest = DailyNotesVerboseQuest:IsChecked()
  DN_Options.itemcount = DailyNotesCounterDisplay:IsChecked()
  DN_Options.dqionly = DailyNotesCounterDQ:IsChecked()
  DN_Options.debug = DailyNotesDebug:IsChecked()

  DN_Options.TT_mobs = DailyNotesTT_Mobs:IsChecked()
  DN_Options.TT_items = DailyNotesTT_Items:IsChecked()
  DN_Options.TT_dialog = DailyNotesTT_Dialog:IsChecked()
  DN_Options.ZBag_Hook = DailyNotesZbagHook:IsChecked()

  DailyNotes.EnableUnitTooltip(DN_Options.TT_mobs)
  DailyNotes.EnableLootMessage(DN_Options.itemcount)

  if DN_Options.itemcount then
    DailyNotesCounterDQ:Enable()
  else
    DailyNotesCounterDQ:Disable()
  end

  if DailyNotesFrame:IsVisible() then
    DailyNotes.UpdateListView()
  end
end



function DailyNotes.Config_Clear()

    StaticPopupDialogs["DN_CONFIG_CLEAR"] = {
        text   = DailyNotes.L.CONFIG_CLEAR,
        button1  = TEXT("ACCEPT"),
        button2  = TEXT("CANCEL"),
        OnAccept = function ()
                DailyNotes.Config_Reset()
                DailyNotes.Config_Load()

                if DailyNotesFrame:IsVisible() then
                  DailyNotes.OnShow()
                end
              end,
        timeout = 0,
        hideOnEscape  = 1
    }

    StaticPopup_Show( "DN_CONFIG_CLEAR" );
end

-----------------------------------------------------------------------
-- interactions
-----------------------------------------------------------------------

function DailyNotes.OnLoad(this)
  this:RegisterEvent("VARIABLES_LOADED")
end

function DailyNotes.OnEvent(event)
  if DailyNotes[event] then DailyNotes[event](arg1) end
end

function DailyNotes.VARIABLES_LOADED()

    SaveVariables("DN_Options")
    SaveVariablesPerCharacter("DN_Options_Of_Char")

    DailyNotes.UpdateOptionsOfOldVersions()

    DailyNotes.RegisterIn3rdPary()

    -- check quest zones (if not european server)
    for id,d in pairs(DailyNotes.DB_Quests) do
        if not GetZoneLocalName(d.zone) then
            DailyNotes.DB_Quests[id]=nil
        end
    end

    -- Register Events
    DailyNotesFrame:RegisterEvent("BAG_ITEM_UPDATE")
    DailyNotesFrame:RegisterEvent("LOADING_END")
    -- these are for tracking the dailies-done-counter
    DailyNotesFrame:RegisterEvent("QUEST_MSG_DAILYRESET")     -- never tested if this event is really fired
    DailyNotesFrame:RegisterEvent("QUEST_MSG_DAILY_OVERDONE") -- never tested if this event is really fired
    DailyNotesFrame:RegisterEvent("RESET_QUESTTRACK") -- it's triggered for everything

    -- enable/disable mob tooltip
    DailyNotes.EnableUnitTooltip(DN_Options.TT_mobs)
    DailyNotes.EnableLootMessage(DN_Options.itemcount)
end

function DailyNotes.RegisterIn3rdPary()
    DailyNotes.RegisterInAddonManager()
    DailyNotes.RegisterInAdvancedAuctionHouse()
    DailyNotes.RegisterInZBag()
end

function DailyNotes.RegisterInAddonManager()

    if AddonManager then
        local addon = {
            name         = "DailyNotes",
            description  = "Detailed information for daily-quest items.",
            author       = "McBen, rv55",
            icon         = "interface/AddOns/dailynotes/icon.dds",
            category     = "Inventory",
            slashCommand = "/dn",
            version      = DailyNotes.version,
            configFrame  = DailyNotesFrame,
            miniButton   = DailyNotesMiniButton,
        }
        AddonManager.RegisterAddonTable(addon)
    end

end

function DailyNotes.RegisterInAdvancedAuctionHouse()
    if AAHFunc and AAHFunc.FiltersRegister then
        AAHFunc.FiltersRegister({coms = "$dn,$daily", func = DailyNotes.AAH_Filter})
    end
end

function DailyNotes.AAH_Filter(param, auction_entry)
    if not auction_entry or not auction_entry.itemid then return true end

    local _,quest_data = DailyNotes.FindQuestForItem(auction_entry.itemid)
    if not quest_data or quest_data.type==3 then return false end

    local min_xp =param[2] and tonumber(string.match(param[2], "%d+"))
    local max_lvl=param[3] and tonumber(string.match(param[3], "%d+"))

    if min_xp and quest_data.rew[1] < min_xp then return false end
    if max_lvl and quest_data.rlvl >= max_lvl then return false end

    return true
end

function DailyNotes.RegisterInZBag()

    if ZBag_IsDailyItem then

        local ori_fct = ZBag_IsDailyItem
        ZBag_IsDailyItem = function (index)

            if not DN_Options.ZBag_Hook then   return ori_fct(index)  end

            if ori_fct(index) then return true  end

            return DailyNotes.FindQuestForItem( Nyx.GetBagItem(index) or -1 )
        end
    end

end


function DailyNotes.UpdateOptionsOfOldVersions()

  local version= Nyx.VersionToNumber( DN_Options.version )

  -- pre 1.7.0
  if version<10700 then
    -- delete old quest
    for name,data in pairs(DN_Options) do
      if type(data)=="table" then
        if string.find(name,".*%s+(%w*):(%w*)$") then
          DN_Options[name]=nil
        end
      end
    end

    -- convert names to ID
    for n,data in pairs(DN_Options) do
      if type(data)=="table" and data.aq_accept then

        data.FilterZoneName=nil
        local new_data={}
        for name,val in pairs(data.aq_accept) do
          local QID = DailyNotes.FindQuestByName(name)
          if val and QID then
            new_data[QID]=1
          end
        end
        data.aq_accept=new_data
      end
    end
  end

  if version<10806 then
    DN_Options.disabled = nil  -- removed in 1.8.6
  end

  if version<10900 then
    DN_Options.QuestsText = nil     -- removed in 1.9
    DN_Options.verbose_quest = true -- reset to true
    DN_Options.TT_mobs  = true   -- new
    DN_Options.TT_items = true   -- new
    DN_Options.TT_dialog= true   -- new
  end

  if version<10903 then
    DN_Options.itemcount = not DN_Options.noitemcount   -- renamed
    DN_Options.noitemcount = nil
  end

  if version<40001 then
    DN_Options.ZBag_Hook = true -- new
  end

  DN_Options.version = DailyNotes.version
end


function DailyNotes.LOADING_END()

    -- Copy old data into per_char options
    local realm = Nyx.GetCurrentRealm()
    if not DN_Options_Of_Char[realm] or Nyx.TableSize(DN_Options_Of_Char[realm])==0 then
        local charname = Nyx.GetUniqueCharname()
        DN_Options_Of_Char[realm] = DN_Options[charname] or {}
        DN_Options[charname] = nil
    end
    DN_Char_Options = DN_Options_Of_Char[realm]

    --[===[@debug@
    if DailyNotes.TestDB then
        DailyNotes.TestDB()
        DailyNotes.TestDB=nil
    end
    --@end-debug@]===]
end


function DailyNotes.BAG_ITEM_UPDATE()
  if DailyNotesFrame:IsVisible() then
    DailyNotes.DoUpdate=0.5
  end
end

function DailyNotes.OnUpdate(elapsedTime)
  if DailyNotes.DoUpdate then
    DailyNotes.DoUpdate = DailyNotes.DoUpdate - elapsedTime
    if DailyNotes.DoUpdate <0 then
      DailyNotes.DoUpdate = nil
      DailyNotes.UpdateList()
   end
  end
end

function DailyNotes.sw_out(name,value)
  o = "disabled"
  if value then o="enabled" end
  DEFAULT_CHAT_FRAME:AddMessage("DailyNotes "..name..": ".. o)
end



SLASH_DailyNotes1 = "/dailynotes"
SLASH_DailyNotes2 = "/dn"
SlashCmdList["DailyNotes"] = function (editBox, msg)

    -- No command means show the UI
    if ( not msg or msg == "" ) then
        ToggleUIFrame(DailyNotesFrame)
        return;
    end

    local cmds={}
    for cm in string.gmatch(string.lower(msg), "%a+") do table.insert(cmds,cm) end

    local help = nil
    for i = 1,table.getn(cmds) do
        fl = string.sub(cmds[i],1,1)
        if fl == 'd' then
            DN_Options.debug = not DN_Options.debug
            DailyNotes.sw_out("DEBUG",DN_Options.debug)

        elseif fl == 'a' then
            DN_Char_Options.autoquest = not DN_Char_Options.autoquest
            DailyNotes.sw_out("Auto-Quest-Taker",DN_Options[Nyx.GetUniqueCharname()].autoquest)

        elseif fl == 'v' then
            DN_Options.verbose = not DN_Options.verbose
            DailyNotes.sw_out("VERBOSE",DN_Options.verbose)
        else
            DEFAULT_CHAT_FRAME:AddMessage("|cffff0000Unknown switch "..cmds[i])
            help = true
         end
    end

    if help then
        DEFAULT_CHAT_FRAME:AddMessage("|cffa0a0ffDailyNotes |cffe0e0e0"..DailyNotes.version)
        DEFAULT_CHAT_FRAME:AddMessage("switches are:")
        DEFAULT_CHAT_FRAME:AddMessage(" d for debug")
        DEFAULT_CHAT_FRAME:AddMessage(" a for auto-quest-taker")
        DEFAULT_CHAT_FRAME:AddMessage(" v for more information")
    end

end








