MAX_PARTY_BUFFS = 8;
MAX_PARTY_DEBUFFS = 8;
NUM_PARTY_MEMBERS = 6;

function SUI_PartyBuffButton_Update(this)
	local frameName = this:GetName();
	for i = 1, MAX_PARTY_BUFFS do
		local button = getglobal(frameName.."BuffButton"..i);
		local name, texture, count = UnitBuff(this.unit, i);
		if (name) then
			if (count < 1) then
				count = "";
			end
			getglobal(button:GetName().."Icon"):SetTexture(texture);
			getglobal(button:GetName().."Count"):SetText(count);
			button:Show();
		else
			button:Hide();
		end
	end
	
	for i = 1, MAX_PARTY_DEBUFFS do
		local button = getglobal(frameName.."DebuffButton"..i);
		local name, texture, count = UnitDebuff(this.unit, i);
		if (name) then
			if (count < 1) then
				count = "";
			end
			getglobal(button:GetName().."Icon"):SetTexture(texture);
			getglobal(button:GetName().."Count"):SetText(count);
			button:Show();
		else
			button:Hide();
		end
	end

	if (this.debufAnchorToBuf) then
		local buffButton = getglobal(frameName.."BuffButton"..1);
		local debuffButton = getglobal(frameName.."DebuffButton"..1);
		if (buffButton:IsVisible()) then
			debuffButton:ClearAllAnchors();
			debuffButton:SetAnchor("TOPLEFT", "BOTTOMLEFT", buffButton, 0, 0);
		else
			debuffButton:ClearAllAnchors();
			debuffButton:SetAnchor("TOPLEFT", "TOPLEFT", buffButton, 0, 1);
		end
	end
end

-- function SUI_PartyBuffButton_OnEnter(this)
-- end

function SUI_PartyMemberFrame_UpdateMember(this)
	SUI_UnitFrame_Update(this);
	local id = this:GetID();
	
	if (GetNumRaidMembers() > 0 and g_RaidFrameOption.HidePartyInRaid) then
		this:Hide();
		if (this.target) then
			this.target:Hide();
		end
		return;
	end

	local name, online = GetPartyMember(id);
	if (name) then
		local portrait = getglobal(this:GetName().."Portrait" );

		if (online) then
			this:SetAlpha(1.0);
			portrait:Show();
		else
			this:SetAlpha(0.5);
			portrait:Hide();
		end
		
		SUI_PartyBuffButton_Update(this);
		this:Show();
	else
		this:Hide();
		if( this.target )then
			this.target:Hide();
		end
	end
	SUI_PartyMemberFrame_UpdateLeader(this);
	SUI_PartyMemberFrame_UpdateLooter(this);
end

function SUI_PartyMemberFrame_UpdateLeader(this)
	local id = this:GetID();
	local icon = getglobal(this:GetName().."BorderLeaderIcon");
	if (GetPartyLeaderIndex() == id) then
		icon:Show();
	else
		icon:Hide();
	end
end

function SUI_PartyMemberFrame_UpdateLooter(this)
	if (UnitIsMasterLooter("party"..this:GetID())) then
		getglobal(this:GetName().."BorderLooterIcon"):Show();
	else
		getglobal(this:GetName().."BorderLooterIcon"):Hide();
	end
end

function SUI_PartyMemberFrame_OnLoad(this)
	local prefix = this:GetName();	
	local nsub = string.len("SUI_UnitFrame_") + 1;
	local name = this:GetName();
	this.unit = string.sub(name, nsub);

	SUI_UnitFrame_Initialize(this, this.unit, 
		getglobal(prefix.."Name"), getglobal(prefix.."Portrait"),
		getglobal(prefix.."HealthBar"), getglobal(prefix.."ManaBar"), 
		getglobal(prefix.."SkillBar"), getglobal(prefix.."RaidTargetFrame"));
		
	this:RegisterForClicks("LeftButton", "RightButton");
	this:RegisterEvent("PARTY_MEMBER_CHANGED");
	this:RegisterEvent("PARTY_LEADER_CHANGED");
	this:RegisterEvent("UNIT_CLASS_CHANGED");
	this:RegisterEvent("UNIT_BUFF_CHANGED");
	this:RegisterEvent("RAID_TARGET_CHANGED");
	this:RegisterEvent("LOOT_METHOD_CHANGED");

	this.AutoShow = 1;
	SUI_PartyMemberFrame_UpdateMember(this);
end

function SUI_PartyMemberFrame_OnEvent(this, event, arg1, arg2)
	SUI_UnitFrame_OnEvent(this, event, arg1, arg2);
	
	if (event == "PARTY_MEMBER_CHANGED") then
		SUI_PartyMemberFrame_UpdateMember(this);
		return;
	end
	if (event == "PARTY_LEADER_CHANGED") then
		SUI_PartyMemberFrame_UpdateLeader(this);
		return;
	end
	if ( event == "UNIT_BUFF_CHANGED" ) then
		if (arg1 == this.unit) then
			SUI_PartyBuffButton_Update(this);
		end
		return;
	end

	if (event == "UNIT_TARGET_CHANGED") then
		if (arg1 == this.owner) then
			if (this.AutoShow)then
				if (UnitExists(this.unit)) then
					this:Show();
				else
					this:Hide();
				end
			end
			SUI_PartyBuffButton_Update(this);
			SUI_UnitFrame_Update(this);
		end
	end
	if (event == "LOOT_METHOD_CHANGED") then
		SUI_PartyMemberFrame_UpdateLooter(this);
	end
	if (event == "UNIT_CLASS_CHANGED") then
		return;
	end
end

function SUI_PartyMemberFrame_OnClick(this, key)
	local unit = this.unit;
	if (key == "LBUTTON") then
		if (SpellIsTargeting()) then
			SpellTargetUnit(unit);
		else
			TargetUnit(unit);
		end
	else
		ToggleDropDownMenu(getglobal(this:GetName().."DropDown"), 1, nil, this:GetName(), -2, 0);
	end
end

function SUI_PartyFrameDropDown_OnLoad(this)
	UIDropDownMenu_Initialize(this, SUI_PartyFrameDropDown_Show, "MENU");
end

function SUI_PartyFrameDropDown_Show()
	local dropdownMenu = getglobal(UIDROPDOWNMENU_OPEN_MENU);

	local frame = dropdownMenu:GetParent();
	local unit = frame.unit;
	local menu;

	if (not UnitExists(unit))then
		return ;
	end

	if (UnitIsUnit(unit, "player")) then
		menu = "SELF";
	elseif (UnitIsUnit(unit, "pet")) then
		menu = "PET";
	elseif (UnitIsPlayer(unit) ) then
		if (UnitInRaid(unit)) then
		
			if (frame.mode == DF_PARTYFRAME_OPTION)then
				menu = "RAIDOPTION";	
			else
				menu = "RAID";	
			end
	
		elseif (UnitInParty(unit)) then
			menu = "PARTY";
		else
			menu = "PLAYER";
		end
	else
		menu = "NPC";
	end

	if (menu) then
		UnitPopop_ShowMenu(dropdownMenu, menu, frame.unit);
	end
end

-- function SUI_PartyMemberFrame_OnEnter(this)
-- end

-- function SUI_PartyMemberFrame_OnLeave(this)
-- end