-- Garbage collection routines for ntBuff
-- Basic idea from Yamabuki's Advanced GC


function ntbGC_OnLoad(this)
	this:RegisterEvent("ADDNEW_QUESTBOOK");
	this:RegisterEvent("AUCTION_CLOSE");
	this:RegisterEvent("AUCTION_OPEN");
	this:RegisterEvent("BANK_CLOSE");
	this:RegisterEvent("BANK_OPEN");
	this:RegisterEvent("PLAYER_LEAVE_COMBAT");
	this:RegisterEvent("QUEST_COMPLETE");
	this:RegisterEvent("STORE_OPEN");
	this:RegisterEvent("STORE_CLOSE");
	this:RegisterEvent("CHAT_MSG_SYSTEM_GET");
	this:RegisterEvent("CHAT_MSG_SYSTEM_VALUE");

end

function ntbGClog(txt)
	if ntBuff.GCdebug then
		if ntBuff.GCchannelNum then
			getglobal("ChatFrame"..ntBuff.GCchannelNum):AddMessage(txt)
		else
			DEFAULT_CHAT_FRAME:AddMessage(txt)
		end
	end
end

function ntbGC_OnEvent(this, event)
	if ntBuff.GCenable and not chkAGC() then
		ntbGC_stepGC()
	end
end

function chkAGC()
	if AGC then
		if AGC.enabled then
			ntbGClog("AGC enabled")
			return true
		end
	end
	return false
end

local lastGC=os.time()
function ntbGC_stepGC()
	if os.time()>lastGC+60 and not chkAGC() then
		ntBuff.GCenable=false
		local i, before, after, isfinished = 0, 0, 0, false;
		local loop = ntBuff["GCloop"]
		local size = ntBuff["GCsize"]
		before = collectgarbage("count");
		after = before;
		if loop then 
			if loop>50 then loop=50 end
		else
			loop=10
		end
		
		if size then 
			if size>200 then size=200 end
		else
			size=50
		end

		while i<loop and not isfinished and before <= after + size do 
			i=i+1
			isfinished = collectgarbage("step", size);
			after = collectgarbage("count");
			if (isfinished) then
				ntbGClog("Step GC finished. ".. math.floor(before - after) .. " KB released by " .. i .. " steps.");
			elseif (before - after > size) or (i>=loop) then
				ntbGClog("Step GC released ".. math.floor(before - after) .. " KB by " .. i .. " steps.");
			end
		end	
		ntBuff.GCenable=true
		lastGC=os.time()
	end
end

function ntbGC_FullGC()
	if ntBuff.GCenable then
		local before = collectgarbage("count");
		collectgarbage("collect");
		ntbGClog("Full GC released ".. tostring(math.floor(((before-collectgarbage("count"))/1024*10)+0.5)/10).." MB.")
		lastGC=os.time()
	end
end


-- hooks to revert to the original settings before log out

local org_StaticPopup_Show = StaticPopup_Show;
function StaticPopup_Show(...)
	local _arg1 = ...;			-- just first arg needed
	
	if (_arg1 == "QUIT" or _arg1 == "CAMP") then
		collectgarbage("setpause", 200)
		collectgarbage("setstepmul", 200)
		collectgarbage("collect")
		collectgarbage("stop")
		ntBuff.GCenable=false
	end
	return org_StaticPopup_Show(...);	-- pass full args to original func
end

local org_Logout = Logout;
function Logout()
	collectgarbage("setpause", 200)
	collectgarbage("setstepmul", 200)
	collectgarbage("collect")
	collectgarbage("stop")
	ntBuff.GCenable=false
	return org_Logout();
end

local org_QuitGame=QuitGame
function QuitGame()
	collectgarbage("setpause", 200)
	collectgarbage("setstepmul", 200)
	collectgarbage("collect")
	collectgarbage("stop")
	ntBuff.GCenable=false
	return org_QuitGame();
end


local org_CancelLogout = CancelLogout;
function CancelLogout()
	collectgarbage("setpause", ntBuff["GCpause"]+100);
	collectgarbage("setstepmul", ntBuff["GCstepmul"]+200);
	collectgarbage("restart")
	ntBuff.GCenable=true
	return org_CancelLogout();
end


